# -*- coding: utf-8 -*- #
# Copyright 2022 Google LLC. All Rights Reserved.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#    http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
"""Cloud Workstations configs API utilities."""


import re

from googlecloudsdk.api_lib.util import waiter
from googlecloudsdk.api_lib.workstations.util import GetClientInstance
from googlecloudsdk.api_lib.workstations.util import GetMessagesModule
from googlecloudsdk.api_lib.workstations.util import VERSION_MAP
from googlecloudsdk.calliope import base
from googlecloudsdk.core import log
from googlecloudsdk.core import resources
import six


IMAGE_URL_MAP = {
    'base-image': (
        '{location}-docker.pkg.dev/cloud-workstations-images/predefined/base:latest'
    ),
    'clion': (
        '{location}-docker.pkg.dev/cloud-workstations-images/predefined/clion:latest'
    ),
    'codeoss': (
        '{location}-docker.pkg.dev/cloud-workstations-images/predefined/code-oss:latest'
    ),
    'codeoss-cuda': (
        '{location}-docker.pkg.dev/cloud-workstations-images/predefined/code-oss-cuda:latest'
    ),
    'goland': (
        '{location}-docker.pkg.dev/cloud-workstations-images/predefined/goland:latest'
    ),
    'intellij': (
        '{location}-docker.pkg.dev/cloud-workstations-images/predefined/intellij-ultimate:latest'
    ),
    'phpstorm': (
        '{location}-docker.pkg.dev/cloud-workstations-images/predefined/phpstorm:latest'
    ),
    'pycharm': (
        '{location}-docker.pkg.dev/cloud-workstations-images/predefined/pycharm:latest'
    ),
    'rider': (
        '{location}-docker.pkg.dev/cloud-workstations-images/predefined/rider:latest'
    ),
    'rubymine': (
        '{location}-docker.pkg.dev/cloud-workstations-images/predefined/rubymine:latest'
    ),
    'webstorm': (
        '{location}-docker.pkg.dev/cloud-workstations-images/predefined/webstorm:latest'
    ),
}

BOOST_CONFIG_MAP = {
    'id': 'id',
    'machine-type': 'machineType',
    'pool-size': 'poolSize',
    'boot-disk-size': 'bootDiskSizeGb',
    'enable-nested-virtualization': 'enableNestedVirtualization',
    'reservation-affinity': 'reservationAffinity',
}

RESERVATION_AFFINITY_MAP = {
    'key': 'key',
    'consume-reservation-type': 'consumeReservationType',
    'values': 'values',
}


class Configs:
  """The Configs set of Cloud Workstations API functions."""

  def __init__(self, release_track=base.ReleaseTrack.BETA):
    self.api_version = VERSION_MAP.get(release_track)
    self.client = GetClientInstance(release_track)
    self.messages = GetMessagesModule(release_track)
    self._service = (
        self.client.projects_locations_workstationClusters_workstationConfigs
    )

  def Create(self, args):  # pylint: disable=invalid-name
    """Create a new workstation configuration.

    Args:
      args: argparse.Namespace, The arguments that this command was invoked
        with.

    Returns:
      Workstation configuration that was created.
    """
    config_name = args.CONCEPTS.config.Parse().RelativeName()
    parent = config_name.split('/workstationConfigs/')[0]
    location = re.search(r'/locations/(?P<location>[^/]+)/', config_name).group(
        'location'
    )
    config_id = config_name.split('/workstationConfigs/')[1]

    config = self.messages.WorkstationConfig()
    config.name = config_name
    config.idleTimeout = '{}s'.format(args.idle_timeout)
    config.runningTimeout = '{}s'.format(args.running_timeout)
    if args.labels:
      config.labels = self.messages.WorkstationConfig.LabelsValue(
          additionalProperties=[
              self.messages.WorkstationConfig.LabelsValue.AdditionalProperty(
                  key=key, value=value
              )
              for key, value in sorted(six.iteritems(args.labels))
          ]
      )
    config.disableTcpConnections = args.disable_tcp_connections
    config.maxUsableWorkstations = args.max_usable_workstations_count

    # GCE Instance Config
    config.host = self.messages.Host()
    config.host.gceInstance = self.messages.GceInstance()
    config.host.gceInstance.machineType = args.machine_type
    config.host.gceInstance.serviceAccount = args.service_account
    if args.service_account_scopes:
      config.host.gceInstance.serviceAccountScopes = args.service_account_scopes
    if args.network_tags:
      config.host.gceInstance.tags = args.network_tags
    config.host.gceInstance.poolSize = args.pool_size
    config.host.gceInstance.disablePublicIpAddresses = (
        args.disable_public_ip_addresses
    )
    config.host.gceInstance.shieldedInstanceConfig = (
        self.messages.GceShieldedInstanceConfig(
            enableSecureBoot=args.shielded_secure_boot,
            enableVtpm=args.shielded_vtpm,
            enableIntegrityMonitoring=args.shielded_integrity_monitoring,
        )
    )
    config.host.gceInstance.confidentialInstanceConfig = (
        self.messages.GceConfidentialInstanceConfig(
            enableConfidentialCompute=args.enable_confidential_compute
        )
    )
    config.host.gceInstance.enableNestedVirtualization = (
        args.enable_nested_virtualization
    )
    config.host.gceInstance.bootDiskSizeGb = args.boot_disk_size
    if args.IsSpecified('disable_ssh_to_vm'):
      config.host.gceInstance.disableSsh = args.disable_ssh_to_vm
    else:
      config.host.gceInstance.disableSsh = not args.enable_ssh_to_vm
    if args.accelerator_type and args.accelerator_count:
      accelerators = [
          self.messages.Accelerator(
              type=args.accelerator_type,
              count=args.accelerator_count,
          )
      ]
      config.host.gceInstance.accelerators = accelerators
    if args.startup_script_uri:
      config.host.gceInstance.startupScriptUri = args.startup_script_uri

    if args.instance_metadata:
      instance_metadata_value_message = (
          self.messages.GceInstance.InstanceMetadataValue
      )
      additional_property_message = (
          self.messages.GceInstance.InstanceMetadataValue.AdditionalProperty
      )
      config.host.gceInstance.instanceMetadata = (
          instance_metadata_value_message(
              additionalProperties=[
                  additional_property_message(key=key, value=value)
                  for key, value in args.instance_metadata.items()
              ]
          )
      )

    if self.api_version != VERSION_MAP.get(base.ReleaseTrack.GA):
      config.httpOptions = self.messages.HttpOptions()
      if args.allow_unauthenticated_cors_preflight_requests:
        config.httpOptions.allowedUnauthenticatedCorsPreflightRequests = True
      if args.disable_localhost_replacement:
        config.httpOptions.disableLocalhostReplacement = True

    if (
        self.api_version != VERSION_MAP.get(base.ReleaseTrack.GA)
        and args.boost_config
    ):
      for boost_config in args.boost_config:
        desired_boost_config = self.messages.BoostConfig()
        for key, value in boost_config.items():
          if key == 'accelerator-type' or key == 'accelerator-count':
            desired_boost_config.accelerators = [
                self.messages.Accelerator(
                    type=boost_config.get('accelerator-type', ''),
                    count=boost_config.get('accelerator-count', 0),
                )
            ]
          elif key == 'reservation-affinity':
            desired_reservation_affinity = self.messages.ReservationAffinity()
            for key, value in boost_config.get(
                'reservation-affinity', {}
            ).items():
              if key == 'consume-reservation-type':
                value = self.messages.ReservationAffinity.ConsumeReservationTypeValueValuesEnum(
                    value
                )
              setattr(
                  desired_reservation_affinity,
                  RESERVATION_AFFINITY_MAP.get(key),
                  value,
              )
            desired_boost_config.reservationAffinity = (
                desired_reservation_affinity
            )
          else:
            setattr(desired_boost_config, BOOST_CONFIG_MAP.get(key), value)
        config.host.gceInstance.boostConfigs.append(desired_boost_config)

    if (
        self.api_version != VERSION_MAP.get(base.ReleaseTrack.GA)
        and args.reservation_affinity
    ):
      desired_reservation_affinity = self.messages.ReservationAffinity()
      for key, value in args.reservation_affinity.items():
        if key == 'consume-reservation-type':
          value = self.messages.ReservationAffinity.ConsumeReservationTypeValueValuesEnum(
              value
          )
        setattr(
            desired_reservation_affinity,
            RESERVATION_AFFINITY_MAP.get(key),
            value,
        )
      config.host.gceInstance.reservationAffinity = desired_reservation_affinity

    if args.allowed_ports:
      for port_range in args.allowed_ports:
        desired_port_range = self.messages.PortRange()
        for key, value in port_range.items():
          setattr(desired_port_range, key, value)
        config.allowedPorts.append(desired_port_range)

    # Persistent directory
    if not args.no_persistent_storage:
      pd = self.messages.PersistentDirectory()
      pd.mountPath = '/home'

      use_disk_flags = (
          (args.IsKnownAndSpecified('disk_type'))
          or (args.IsKnownAndSpecified('disk_size'))
          or (args.IsKnownAndSpecified('disk_source_snapshot'))
          or (args.IsKnownAndSpecified('disk_reclaim_policy'))
          or (args.IsKnownAndSpecified('disk_archive_timeout'))
      )

      if use_disk_flags:
        disk_type = (
            args.disk_type if args.IsSpecified('disk_type') else 'pd-standard'
        )
        source_snapshot = (
            args.disk_source_snapshot
            if args.IsSpecified('disk_source_snapshot')
            else None
        )
        disk_size = args.disk_size if args.IsSpecified('disk_size') else 200
        reclaim_policy = (
            args.disk_reclaim_policy
            if args.IsSpecified('disk_reclaim_policy')
            else 'delete'
        )
      else:
        disk_type = args.pd_disk_type
        disk_size = args.pd_disk_size
        source_snapshot = args.pd_source_snapshot
        reclaim_policy = args.pd_reclaim_policy

      # Not all instance types can take Hyperdisks, but this is validated on the
      # backend.
      archive_timeout_str = None
      if (
          args.IsKnownAndSpecified('disk_archive_timeout')
          and args.disk_archive_timeout is not None
      ):
        archive_timeout_str = f'{args.disk_archive_timeout}s'

      if disk_type == 'hyperdisk-balanced-ha':
        pd.gceHd = self.messages.GceHyperdiskBalancedHighAvailability(
            sizeGb=0 if source_snapshot else disk_size,
            reclaimPolicy=(
                self.messages.GceHyperdiskBalancedHighAvailability.ReclaimPolicyValueValuesEnum.RETAIN
                if reclaim_policy == 'retain'
                else self.messages.GceHyperdiskBalancedHighAvailability.ReclaimPolicyValueValuesEnum.DELETE
            ),
            sourceSnapshot=source_snapshot,
            archiveTimeout=archive_timeout_str,
        )
      else:
        pd.gcePd = self.messages.GceRegionalPersistentDisk(
            sizeGb=0 if source_snapshot else disk_size,
            fsType='' if source_snapshot else 'ext4',
            diskType=disk_type,
            reclaimPolicy=(
                self.messages.GceRegionalPersistentDisk.ReclaimPolicyValueValuesEnum.RETAIN
                if reclaim_policy == 'retain'
                else self.messages.GceRegionalPersistentDisk.ReclaimPolicyValueValuesEnum.DELETE
            ),
            sourceSnapshot=source_snapshot,
            archiveTimeout=archive_timeout_str,
        )
      config.persistentDirectories.append(pd)

    # Ephemeral directory
    if args.ephemeral_directory:
      for directory in args.ephemeral_directory:
        pd = self.messages.EphemeralDirectory()
        pd.mountPath = directory.get('mount-path')
        pd.gcePd = self.messages.GcePersistentDisk(
            diskType=directory.get('disk-type'),
            sourceSnapshot=directory.get('source-snapshot'),
            sourceImage=directory.get('source-image'),
            readOnly=directory.get('read-only'),
        )
        config.ephemeralDirectories.append(pd)

    # Container
    config.container = self.messages.Container()
    if args.container_custom_image:
      config.container.image = args.container_custom_image
    elif args.container_predefined_image:
      config.container.image = IMAGE_URL_MAP.get(
          args.container_predefined_image
      ).format(location=location)
    if args.container_command:
      config.container.command = args.container_command
    if args.container_args:
      config.container.args = args.container_args
    if args.container_env:
      env_val = self.messages.Container.EnvValue()
      for key, value in args.container_env.items():
        env_val.additionalProperties.append(
            self.messages.Container.EnvValue.AdditionalProperty(
                key=key, value=value
            )
        )
      config.container.env = env_val
    config.container.workingDir = args.container_working_dir
    config.container.runAsUser = args.container_run_as_user

    # Encryption key
    encryption_key = self.messages.CustomerEncryptionKey()
    if args.kms_key:
      encryption_key.kmsKey = args.kms_key
    if args.kms_key_service_account:
      encryption_key.kmsKeyServiceAccount = args.kms_key_service_account
    config.encryptionKey = encryption_key

    if args.enable_audit_agent:
      config.enableAuditAgent = args.enable_audit_agent

    if args.grant_workstation_admin_role_on_create:
      config.grantWorkstationAdminRoleOnCreate = (
          args.grant_workstation_admin_role_on_create
      )

    if args.replica_zones:
      config.replicaZones = args.replica_zones

    if args.vm_tags:
      tags_val = self.messages.GceInstance.VmTagsValue()
      for key, value in args.vm_tags.items():
        tags_val.additionalProperties.append(
            self.messages.GceInstance.VmTagsValue.AdditionalProperty(
                key=key, value=value
            )
        )
      config.host.gceInstance.vmTags = tags_val

    create_req = self.messages.WorkstationsProjectsLocationsWorkstationClustersWorkstationConfigsCreateRequest(
        parent=parent, workstationConfigId=config_id, workstationConfig=config
    )
    op_ref = self._service.Create(create_req)

    log.status.Print('Create request issued for: [{}]'.format(config_id))

    if args.async_:
      log.status.Print('Check operation [{}] for status.'.format(op_ref.name))
      return op_ref

    op_resource = resources.REGISTRY.ParseRelativeName(
        op_ref.name,
        collection='workstations.projects.locations.operations',
        api_version=self.api_version,
    )
    poller = waiter.CloudOperationPoller(
        self._service, self.client.projects_locations_operations
    )

    result = waiter.WaitFor(
        poller,
        op_resource,
        'Waiting for operation [{}] to complete'.format(op_ref.name),
    )
    log.status.Print('Created configuration [{}].'.format(config_id))

    return result

  def Update(self, args):  # pylint: disable=invalid-name
    """Updates an existing workstation configuration.

    Args:
      args: argparse.Namespace, The arguments that this command was invoked
        with.

    Returns:
      Workstation configuration that was updated.
    """
    config_name = args.CONCEPTS.config.Parse().RelativeName()
    location = re.search(r'/locations/(?P<location>[^/]+)/', config_name).group(
        'location'
    )
    config_id = config_name.split('/workstationConfigs/')[1]

    config = self.messages.WorkstationConfig()
    config.name = config_name
    get_req = self.messages.WorkstationsProjectsLocationsWorkstationClustersWorkstationConfigsGetRequest(
        name=config_name
    )
    old_config = self._service.Get(get_req)
    update_mask = []

    if args.IsSpecified('idle_timeout'):
      config.idleTimeout = '{}s'.format(args.idle_timeout)
      update_mask.append('idle_timeout')

    if args.IsSpecified('running_timeout'):
      config.runningTimeout = '{}s'.format(args.running_timeout)
      update_mask.append('running_timeout')

    if args.IsSpecified('labels'):
      config.labels = self.messages.WorkstationConfig.LabelsValue(
          additionalProperties=[
              self.messages.WorkstationConfig.LabelsValue.AdditionalProperty(
                  key=key, value=value
              )
              for key, value in sorted(six.iteritems(args.labels))
          ]
      )
      update_mask.append('labels')

    if args.IsSpecified('max_usable_workstations_count'):
      config.maxUsableWorkstations = args.max_usable_workstations_count
      update_mask.append('max_usable_workstations')

    if self.api_version != VERSION_MAP.get(base.ReleaseTrack.GA):
      config.httpOptions = self.messages.HttpOptions()
      if args.allow_unauthenticated_cors_preflight_requests:
        config.httpOptions.allowedUnauthenticatedCorsPreflightRequests = True
        update_mask.append(
            'http_options.allowed_unauthenticated_cors_preflight_requests'
        )
      if args.disallow_unauthenticated_cors_preflight_requests:
        config.httpOptions.allowedUnauthenticatedCorsPreflightRequests = False
        update_mask.append(
            'http_options.allowed_unauthenticated_cors_preflight_requests'
        )

      if args.enable_localhost_replacement:
        config.httpOptions.disableLocalhostReplacement = False
        update_mask.append('http_options.disable_localhost_replacement')
      if args.disable_localhost_replacement:
        config.httpOptions.disableLocalhostReplacement = True
        update_mask.append('http_options.disable_localhost_replacement')

    # GCE Instance Config
    config.host = self.messages.Host()
    config.host.gceInstance = self.messages.GceInstance()
    if args.IsSpecified('machine_type'):
      config.host.gceInstance.machineType = args.machine_type
      update_mask.append('host.gce_instance.machine_type')

    if self.api_version != VERSION_MAP.get(
        base.ReleaseTrack.GA
    ) and args.IsSpecified('reservation_affinity'):
      desired_reservation_affinity = self.messages.ReservationAffinity()
      for key, value in args.reservation_affinity.items():
        if key == 'consume-reservation-type':
          value = self.messages.ReservationAffinity.ConsumeReservationTypeValueValuesEnum(
              value
          )
        setattr(
            desired_reservation_affinity,
            RESERVATION_AFFINITY_MAP.get(key),
            value,
        )
      config.host.gceInstance.reservationAffinity = desired_reservation_affinity
      update_mask.append('host.gce_instance.reservation_affinity')

    if args.IsSpecified('service_account'):
      config.host.gceInstance.serviceAccount = args.service_account
      update_mask.append('host.gce_instance.service_account')

    if args.IsSpecified('service_account_scopes'):
      config.host.gceInstance.serviceAccountScopes = args.service_account_scopes
      update_mask.append('host.gce_instance.service_account_scopes')

    if args.IsSpecified('network_tags'):
      config.host.gceInstance.tags = args.network_tags
      update_mask.append('host.gce_instance.tags')

    if args.IsSpecified('pool_size'):
      config.host.gceInstance.poolSize = args.pool_size
      update_mask.append('host.gce_instance.pool_size')

    if args.IsSpecified('disable_public_ip_addresses'):
      config.host.gceInstance.disablePublicIpAddresses = (
          args.disable_public_ip_addresses
      )
      update_mask.append('host.gce_instance.disable_public_ip_addresses')

    if args.IsSpecified('boot_disk_size'):
      config.host.gceInstance.bootDiskSizeGb = args.boot_disk_size
      update_mask.append('host.gce_instance.boot_disk_size_gb')

    if args.IsKnownAndSpecified('disable_ssh_to_vm'):
      config.host.gceInstance.disableSsh = args.disable_ssh_to_vm
      update_mask.append('host.gce_instance.disable_ssh')

    if args.IsKnownAndSpecified('enable_ssh_to_vm'):
      config.host.gceInstance.disableSsh = not args.enable_ssh_to_vm
      update_mask.append('host.gce_instance.disable_ssh')

    if args.IsSpecified('enable_confidential_compute'):
      config.host.gceInstance.confidentialInstanceConfig = (
          self.messages.GceConfidentialInstanceConfig(
              enableConfidentialCompute=args.enable_confidential_compute
          )
      )
      update_mask.append(
          'host.gce_instance.confidential_instance_config.enable_confidential_compute'
      )

    if args.IsSpecified('enable_audit_agent'):
      config.enableAuditAgent = args.enable_audit_agent
      update_mask.append('enable_audit_agent')

    if args.IsSpecified('grant_workstation_admin_role_on_create'):
      config.grantWorkstationAdminRoleOnCreate = (
          args.grant_workstation_admin_role_on_create
      )
      update_mask.append('grant_workstation_admin_role_on_create')

    if args.IsSpecified('disable_tcp_connections'):
      config.disableTcpConnections = args.disable_tcp_connections
      update_mask.append('disable_tcp_connections')

    if args.IsSpecified('enable_tcp_connections'):
      config.disableTcpConnections = not args.enable_tcp_connections
      update_mask.append('disable_tcp_connections')

    if args.IsSpecified('enable_nested_virtualization'):
      config.host.gceInstance.enableNestedVirtualization = (
          args.enable_nested_virtualization
      )
      update_mask.append('host.gce_instance.enable_nested_virtualization')

    if args.startup_script_uri:
      config.host.gceInstance.startupScriptUri = args.startup_script_uri
      update_mask.append('host.gce_instance.startup_script_uri')

    if args.instance_metadata:
      instance_metadata_value_message = (
          self.messages.GceInstance.InstanceMetadataValue
      )
      additional_property_message = (
          self.messages.GceInstance.InstanceMetadataValue.AdditionalProperty
      )
      config.host.gceInstance.instanceMetadata = (
          instance_metadata_value_message(
              additionalProperties=[
                  additional_property_message(key=key, value=value)
                  for key, value in args.instance_metadata.items()
              ]
          )
      )
      update_mask.append('host.gce_instance.instance_metadata')

    # Shielded Instance Config
    gce_shielded_instance_config = self.messages.GceShieldedInstanceConfig()
    if args.IsSpecified('shielded_secure_boot'):
      gce_shielded_instance_config.enableSecureBoot = args.shielded_secure_boot
      update_mask.append(
          'host.gce_instance.shielded_instance_config.enable_secure_boot'
      )

    if args.IsSpecified('shielded_vtpm'):
      gce_shielded_instance_config.enableVtpm = args.shielded_vtpm
      update_mask.append(
          'host.gce_instance.shielded_instance_config.enable_vtpm'
      )

    if args.IsSpecified('shielded_integrity_monitoring'):
      gce_shielded_instance_config.enableIntegrityMonitoring = (
          args.shielded_integrity_monitoring
      )
      update_mask.append(
          'host.gce_instance.shielded_instance_config.enable_integrity_monitoring'
      )

    config.host.gceInstance.shieldedInstanceConfig = (
        gce_shielded_instance_config
    )

    if args.IsSpecified('accelerator_type') or args.IsSpecified(
        'accelerator_count'
    ):
      accelerators = [
          self.messages.Accelerator(
              type=args.accelerator_type,
              count=args.accelerator_count,
          )
      ]
      config.host.gceInstance.accelerators = accelerators
      update_mask.append('host.gce_instance.accelerators')

    if self.api_version != VERSION_MAP.get(
        base.ReleaseTrack.GA
    ) and args.IsSpecified('boost_config'):
      for boost_config in args.boost_config:
        desired_boost_config = self.messages.BoostConfig()
        for key, value in boost_config.items():
          if key == 'accelerator-type' or key == 'accelerator-count':
            desired_boost_config.accelerators = [
                self.messages.Accelerator(
                    type=boost_config['accelerator-type'],
                    count=boost_config['accelerator-count'],
                )
            ]
          elif key == 'reservation-affinity':
            desired_reservation_affinity = self.messages.ReservationAffinity()
            for key, value in boost_config['reservation-affinity'].items():
              if key == 'consume-reservation-type':
                value = self.messages.ReservationAffinity.ConsumeReservationTypeValueValuesEnum(
                    value
                )
              setattr(
                  desired_reservation_affinity,
                  RESERVATION_AFFINITY_MAP.get(key),
                  value,
              )
            desired_boost_config.reservationAffinity = (
                desired_reservation_affinity
            )
          else:
            setattr(desired_boost_config, BOOST_CONFIG_MAP.get(key), value)
        config.host.gceInstance.boostConfigs.append(desired_boost_config)
      update_mask.append('host.gce_instance.boost_configs')

    if args.IsSpecified('allowed_ports'):
      config.allowedPorts = []
      for port_range in args.allowed_ports:
        desired_port_range = self.messages.PortRange()
        for key, value in port_range.items():
          setattr(desired_port_range, key, value)
        config.allowedPorts.append(desired_port_range)
      update_mask.append('allowed_ports')

    # Container
    config.container = self.messages.Container()
    if args.IsSpecified('container_custom_image'):
      config.container.image = args.container_custom_image
      update_mask.append('container.image')
    elif args.IsSpecified('container_predefined_image'):
      config.container.image = IMAGE_URL_MAP.get(
          args.container_predefined_image
      ).format(location=location)
      update_mask.append('container.image')

    if args.IsSpecified('container_command'):
      config.container.command = args.container_command
      update_mask.append('container.command')

    if args.IsSpecified('container_args'):
      config.container.args = args.container_args
      update_mask.append('container.args')

    if args.IsSpecified('container_env'):
      env_val = self.messages.Container.EnvValue()
      for key, value in args.container_env.items():
        env_val.additionalProperties.append(
            self.messages.Container.EnvValue.AdditionalProperty(
                key=key, value=value
            )
        )
      config.container.env = env_val
      update_mask.append('container.env')

    if args.IsSpecified('container_working_dir'):
      config.container.workingDir = args.container_working_dir
      update_mask.append('container.working_dir')

    if args.IsSpecified('container_run_as_user'):
      config.container.runAsUser = args.container_run_as_user
      update_mask.append('container.run_as_user')

    use_disk_type = args.IsKnownAndSpecified(
        'disk_type'
    ) or args.IsKnownAndSpecified('pd_disk_type')
    use_disk_size = args.IsKnownAndSpecified(
        'disk_size'
    ) or args.IsKnownAndSpecified('pd_disk_size')
    use_source_snapshot = args.IsKnownAndSpecified(
        'disk_source_snapshot'
    ) or args.IsKnownAndSpecified('pd_source_snapshot')
    use_disk_archive_timeout = (
        args.IsKnownAndSpecified('disk_archive_timeout')
        and args.disk_archive_timeout is not None
    )
    update_disk = (
        use_disk_type
        or use_disk_size
        or use_source_snapshot
        or use_disk_archive_timeout
    )
    if use_disk_type:
      disk_type: str = (
          args.disk_type
          if args.IsKnownAndSpecified('disk_type')
          else args.pd_disk_type
      )
    else:
      disk_type: str = extract_disk_type(old_config)
    if use_disk_size:
      disk_size: int = (
          args.disk_size
          if args.IsKnownAndSpecified('disk_size')
          else args.pd_disk_size
      )
    else:
      disk_size: int = extract_disk_size(old_config)
    if use_source_snapshot:
      source_snapshot: str = (
          args.disk_source_snapshot
          if args.IsKnownAndSpecified('disk_source_snapshot')
          else args.pd_source_snapshot
      )
    else:
      source_snapshot: str = extract_source_snapshot(old_config)
    if (
        old_config.persistentDirectories
        and old_config.persistentDirectories[0].gcePd
        and disk_type == 'hyperdisk-balanced-ha'
    ):
      log.err.Print("Can't update persistent directory from PD to HD")

    if (
        old_config.persistentDirectories
        and hasattr(old_config.persistentDirectories[0], 'gceHd')
        and old_config.persistentDirectories[0].gceHd
        and disk_type != 'hyperdisk-balanced-ha'
    ):
      log.err.Print("Can't update persistent directory from HD to PD")

    if update_disk:
      update_mask.append('persistent_directories')

    config.persistentDirectories = old_config.persistentDirectories
    if not old_config.persistentDirectories:
      config.persistentDirectories = [self.messages.PersistentDirectory()]

    if use_disk_archive_timeout:
      archive_timeout = f'{args.disk_archive_timeout}s'
    else:
      archive_timeout = extract_archive_timeout(old_config)

    if use_source_snapshot:
      if disk_type == 'hyperdisk-balanced-ha':
        config.persistentDirectories[0].gceHd = (
            self.messages.GceHyperdiskBalancedHighAvailability(
                sizeGb=0,
                sourceSnapshot=source_snapshot,
                archiveTimeout=archive_timeout,
            )
        )
      else:
        config.persistentDirectories[0].gcePd = (
            self.messages.GceRegionalPersistentDisk(
                sizeGb=0,
                fsType='',
                sourceSnapshot=source_snapshot,
                diskType=disk_type,
                archiveTimeout=archive_timeout,
            )
        )
    elif disk_type == 'hyperdisk-balanced-ha':
      config.persistentDirectories[0].gceHd = (
          self.messages.GceHyperdiskBalancedHighAvailability(
              sizeGb=disk_size,
              archiveTimeout=archive_timeout,
          )
      )
    else:
      config.persistentDirectories[0].gcePd = (
          self.messages.GceRegionalPersistentDisk(
              sizeGb=disk_size,
              diskType=disk_type,
              archiveTimeout=archive_timeout,
          )
      )

    if args.IsSpecified('vm_tags'):
      tags_val = self.messages.GceInstance.VmTagsValue()
      for key, value in args.vm_tags.items():
        tags_val.additionalProperties.append(
            self.messages.GceInstance.VmTagsValue.AdditionalProperty(
                key=key, value=value
            )
        )
      config.host.gceInstance.vmTags = tags_val
      update_mask.append('host.gce_instance.vm_tags')

    if not update_mask:
      log.error('No fields were specified.')
      return

    update_req = self.messages.WorkstationsProjectsLocationsWorkstationClustersWorkstationConfigsPatchRequest(
        name=config_name,
        workstationConfig=config,
        updateMask=','.join(update_mask),
    )
    op_ref = self._service.Patch(update_req)

    log.status.Print('Update request issued for: [{}]'.format(config_id))

    if args.async_:
      log.status.Print('Check operation [{}] for status.'.format(op_ref.name))
      return op_ref

    op_resource = resources.REGISTRY.ParseRelativeName(
        op_ref.name,
        collection='workstations.projects.locations.operations',
        api_version=self.api_version,
    )
    poller = waiter.CloudOperationPoller(
        self._service, self.client.projects_locations_operations
    )

    result = waiter.WaitFor(
        poller,
        op_resource,
        'Waiting for operation [{}] to complete'.format(op_ref.name),
    )
    log.status.Print('Updated configuration [{}].'.format(config_id))
    return result


def extract_disk_type(old_config) -> str:
  if old_config.persistentDirectories and getattr(
      old_config.persistentDirectories[0], 'gceHd', False
  ):
    return 'hyperdisk-balanced-ha'
  if old_config.persistentDirectories and getattr(
      old_config.persistentDirectories[0], 'gcePd', False
  ):
    return old_config.persistentDirectories[0].gcePd.diskType
  return ''


def extract_disk_size(old_config) -> int:
  if old_config.persistentDirectories and getattr(
      old_config.persistentDirectories[0], 'gceHd', False
  ):
    return old_config.persistentDirectories[0].gceHd.sizeGb
  if old_config.persistentDirectories and getattr(
      old_config.persistentDirectories[0], 'gcePd', False
  ):
    return old_config.persistentDirectories[0].gcePd.sizeGb
  return 0


def extract_source_snapshot(old_config) -> str:
  if old_config.persistentDirectories and getattr(
      old_config.persistentDirectories[0], 'gceHd', False
  ):
    return old_config.persistentDirectories[0].gceHd.sourceSnapshot
  if old_config.persistentDirectories and getattr(
      old_config.persistentDirectories[0], 'gcePd', False
  ):
    return old_config.persistentDirectories[0].gcePd.sourceSnapshot
  return ''


def extract_archive_timeout(old_config) -> str:
  if old_config.persistentDirectories and getattr(
      old_config.persistentDirectories[0], 'gceHd', False
  ):
    return old_config.persistentDirectories[0].gceHd.archiveTimeout
  if old_config.persistentDirectories and getattr(
      old_config.persistentDirectories[0], 'gcePd', False
  ):
    return old_config.persistentDirectories[0].gcePd.archiveTimeout
  return None



# -*- coding: utf-8 -*- #
# Copyright 2026 Google LLC. All Rights Reserved.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#    http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
"""Command to SSH into a Cloud Run instance."""

from googlecloudsdk.api_lib.run import ssh as run_ssh
from googlecloudsdk.calliope import base
from googlecloudsdk.command_lib.run import ssh_command


@base.ReleaseTracks(base.ReleaseTrack.ALPHA)
@base.Hidden
@base.DefaultUniverseOnly
class Ssh(ssh_command.BaseSshCommand):
  """SSH into an instance."""

  detailed_help = {
      'DESCRIPTION': (
          """\
          Starts a secure, interactive shell session with a Cloud Run instance.
          """
      ),
      'EXAMPLES': (
          """\
          To start an interactive shell session with a Cloud Run instance:

              $ {command} my-instance
          """
      ),
  }

  @classmethod
  def Args(cls, parser):
    cls.AddBaseArgs(parser)
    # Add the instance name as a required positional argument.
    parser.add_argument(
        'instance',
        help='The name of the instance to SSH into.',
    )

  def Run(self, args):
    """Connect to a running Cloud Run instance deployment."""
    # For instances, the positional 'instance' is the deployment name.
    self.RunSsh(args, run_ssh.Ssh.WorkloadType.INSTANCE, args.instance)



# -*- coding: utf-8 -*- #
# Copyright 2026 Google LLC. All Rights Reserved.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#    http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
"""Utilities for detecting AI agents in the environment."""

import os

from googlecloudsdk.core.util import encoding

# LINT.IfChange
AI_AGENTS = [
    ('ANTIGRAVITY_AGENT', 'antigravity'),
    ('CLAUDECODE', 'claude_code'),
    ('CLINE_ACTIVE', 'cline'),
    ('CODEX_SANDBOX', 'codex_cli'),
    ('CURSOR_AGENT', 'cursor'),
    ('GEMINI_CLI', 'gemini_cli'),
    ('OPENCODE', 'open_code'),
    ('ANDROID_STUDIO_AGENT', 'android_studio_agent'),
    ('KIRO_AGENT_PATH', 'kiro'),
]
# LINT.ThenChange(../../../../../cloud/helix/clients/cli/bq_utils.py)


def DetectAIAgent():
  """Detects the AI agent based on environment variables.

  Returns:
    str, The name of the AI agent or None.
  """
  for env_var, agent_name in AI_AGENTS:
    if encoding.GetEncodedValue(os.environ, env_var):
      return agent_name
  return None



# -*- coding: utf-8 -*- #
# Copyright 2018 Google LLC. All Rights Reserved.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#    http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
"""Cloud Run CSI metric names."""


# Reserved CSI metric prefix for serverless
_SERVERLESS_PREFIX = 'serverless_'

# Time to create a configuration
CREATE_CONFIGURATION = _SERVERLESS_PREFIX + 'create_configuration'

# Time to create a domain mapping
CREATE_DOMAIN_MAPPING = _SERVERLESS_PREFIX + 'create_domain_mapping'

# Time to create a route
CREATE_ROUTE = _SERVERLESS_PREFIX + 'create_route'

# Time to create a service
CREATE_SERVICE = _SERVERLESS_PREFIX + 'create_service'

# Time to submit a build to Cloud Build and wait for its completion
CREATE_BUILD = _SERVERLESS_PREFIX + 'submit_build'

# Time to delete a domain mapping
DELETE_DOMAIN_MAPPING = _SERVERLESS_PREFIX + 'delete_domain_mapping'

# Time to delete a revision
DELETE_REVISION = _SERVERLESS_PREFIX + 'delete_revision'

# Time to delete a service
DELETE_SERVICE = _SERVERLESS_PREFIX + 'delete_service'

# Time to get a configuration
GET_CONFIGURATION = _SERVERLESS_PREFIX + 'get_configuration'

# Time to get a domain mapping
GET_DOMAIN_MAPPING = _SERVERLESS_PREFIX + 'get_domain_mapping'

# Time to list domain mappings
LIST_DOMAIN_MAPPINGS = _SERVERLESS_PREFIX + 'list_domain_mappings'

# Time to get a revision
GET_REVISION = _SERVERLESS_PREFIX + 'get_revision'

# Time to get a route
GET_ROUTE = _SERVERLESS_PREFIX + 'get_route'

# Time to get a service
GET_SERVICE = _SERVERLESS_PREFIX + 'get_service'

# Time to list configurations
LIST_CONFIGURATIONS = _SERVERLESS_PREFIX + 'list_configurations'

# Time to list revisions
LIST_REVISIONS = _SERVERLESS_PREFIX + 'list_revisions'

# Time to list routes
LIST_ROUTES = _SERVERLESS_PREFIX + 'list_routes'

# Time to list services
LIST_SERVICES = _SERVERLESS_PREFIX + 'list_services'

# Time to update a configuration
UPDATE_CONFIGURATION = _SERVERLESS_PREFIX + 'update_configuration'

# Time to update a service
UPDATE_SERVICE = _SERVERLESS_PREFIX + 'update_service'

# Time to create a job
CREATE_JOB = _SERVERLESS_PREFIX + 'create_job'

# Time to update a job
UPDATE_JOB = _SERVERLESS_PREFIX + 'update_job'

# Time to get a job
GET_JOB = _SERVERLESS_PREFIX + 'get_job'

# Time to get an execution
GET_EXECUTION = _SERVERLESS_PREFIX + 'get_execution'

# Time to get a task
GET_TASK = _SERVERLESS_PREFIX + 'get_task'

# Time to list jobs
LIST_JOBS = _SERVERLESS_PREFIX + 'list_jobs'

# Time to list executions
LIST_EXECUTIONS = _SERVERLESS_PREFIX + 'list_executions'

# Time to list tasks
LIST_TASKS = _SERVERLESS_PREFIX + 'list_tasks'

# Time to delete a job
DELETE_JOB = _SERVERLESS_PREFIX + 'delete_job'

# Time to delete an execution
DELETE_EXECUTION = _SERVERLESS_PREFIX + 'delete_execution'

# Time to cancel an execution
CANCEL_EXECUTION = _SERVERLESS_PREFIX + 'cancel_execution'

# Time to run a job
RUN_JOB = _SERVERLESS_PREFIX + 'run_job'

# Time to get an instance
GET_INSTANCE = _SERVERLESS_PREFIX + 'get_instance'

# Time to list instances
LIST_INSTANCES = _SERVERLESS_PREFIX + 'list_instances'

# Time to create an instance
CREATE_INSTANCE = _SERVERLESS_PREFIX + 'create_instance'

# Time to update an instance
UPDATE_INSTANCE = _SERVERLESS_PREFIX + 'update_instance'

# Time to delete an instance
DELETE_INSTANCE = _SERVERLESS_PREFIX + 'delete_instance'

# Time to stop an instance
STOP_INSTANCE = _SERVERLESS_PREFIX + 'stop_instance'

# Time to start an instance
START_INSTANCE = _SERVERLESS_PREFIX + 'start_instance'

# Time to wait for an operation
WAIT_OPERATION = _SERVERLESS_PREFIX + 'wait_operation'

# Time to create a worker pool
CREATE_WORKER_POOL = _SERVERLESS_PREFIX + 'create_worker_pool'

# Time to update a worker pool
UPDATE_WORKER_POOL = _SERVERLESS_PREFIX + 'update_worker_pool'

# Time to delete a worker pool
DELETE_WORKER_POOL = _SERVERLESS_PREFIX + 'delete_worker_pool'

# Time to list worker pools
LIST_WORKER_POOLS = _SERVERLESS_PREFIX + 'list_worker_pools'

# Time to get a worker pool
GET_WORKER_POOL = _SERVERLESS_PREFIX + 'get_worker_pool'

# Time to delete a worker pool revision
DELETE_WORKER_POOL_REVISION = _SERVERLESS_PREFIX + 'delete_worker_pool_revision'

# Time to list worker pool revisions
LIST_WORKER_POOL_REVISIONS = _SERVERLESS_PREFIX + 'list_worker_pool_revisions'

# Time to get a worker pool revision
GET_WORKER_POOL_REVISION = _SERVERLESS_PREFIX + 'get_worker_pool_revision'

# Custom metric for the region of the service being deployed
CUSTOM_METRIC_REGION = _SERVERLESS_PREFIX + 'region'

# Custom metric for the base image of the service being deployed
CUSTOM_METRIC_BASE_IMAGE = _SERVERLESS_PREFIX + 'base_image'



# -*- coding: utf-8 -*- #
# Copyright 2021 Google LLC. All Rights Reserved.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#    http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
"""Replicate a certificate template to multiple regions."""


from apitools.base.py import exceptions as api_exceptions
from googlecloudsdk.api_lib.privateca import base as privateca_base
from googlecloudsdk.api_lib.privateca import locations
from googlecloudsdk.api_lib.privateca import request_utils
from googlecloudsdk.calliope import arg_parsers
from googlecloudsdk.calliope import base
from googlecloudsdk.command_lib.privateca import operations
from googlecloudsdk.command_lib.privateca import resource_args
from googlecloudsdk.core import log


class ReplicationError(Exception):
  """Represents an error that occurred while replicating a resource to a given location."""

  def __init__(self, location, message):
    self._message = 'Failed to replicate to location [{}]: {}'.format(
        location, message)
    super(ReplicationError, self).__init__(self._message)

  def __str__(self):
    return self._message


@base.ReleaseTracks(base.ReleaseTrack.GA)
class Replicate(base.SilentCommand):
  """Replicate a certificate template to multiple locations."""

  detailed_help = {
      'DESCRIPTION':
          'Replicate a certificate template to multiple locations.',
      'EXAMPLES':
          """\
      To replicate a certificate templates to all supported locations, run:

        $ {command} my-template --location=us-west1 --all-locations

      To replicate a certificate template to 'us-west2' and 'us-east1', run:

        $ {command} my-template --location=us-west1 --target-locations=us-west2,us-east1

      To overwrite existing templates with the same resource ID in the target
      locations, use the --overwrite flag:

        $ {command} my-template --location=us-west1 --target-locations=us-west2,us-east1 --overwrite

      To continue replicating templates in other locations in the event of a
      failure in one or more locations, use the --continue-on-error flag:

        $ {command} my-template --location=us-west1 --all-locations --continue-on-error""",
  }

  @staticmethod
  def Args(parser):
    resource_args.AddCertificateTemplatePositionalResourceArg(
        parser, 'to replicate')

    target_locations_group = base.ArgumentGroup(
        mutex=True,
        required=True,
        help='Specify where the certificate template should be replicated.'
    ).AddToParser(parser)

    base.Argument(
        '--all-locations',
        action='store_const',
        const=True,
        help='Replicate this template to all supported locations.').AddToParser(
            target_locations_group)
    base.Argument(
        '--target-locations',
        help='Replicate this template to the given locations.',
        type=arg_parsers.ArgList(element_type=lambda x: str(x).strip()),
        metavar='LOCATION',
    ).AddToParser(target_locations_group)

    base.Argument(
        '--overwrite',
        help=('Overwrite any existing templates with the same name, '
              'if they exist.'),
        action='store_const',
        const=True,
        default=False).AddToParser(parser)
    base.Argument(
        '--continue-on-error',
        help=('Continue replicating the template to other locations '
              'even if an error is encountered. If this is set, an '
              'error in one location will be logged but will not '
              'prevent replication to other locations.'),
        action='store_const',
        const=True,
        default=False).AddToParser(parser)

  def _CreateOrUpdateTemplate(self, project, location, template_id, template,
                              overwrite):
    """Returns an LRO for a Create or Update operation for the given template.

    Args:
      project: str, the project ID or number for the new template.
      location: str, the location for the new template.
      template_id: str, the resource ID for the new template.
      template: object, the body of the new template.
      overwrite: bool, whether to overwrite existing templates with the same ID.

    Raises:
      ReplicationError, if the template could not be replicated to this
      location.
    """
    parent = 'projects/{}/locations/{}'.format(project, location)
    resource_name = '{}/certificateTemplates/{}'.format(parent, template_id)
    try:
      return self.client.projects_locations_certificateTemplates.Create(
          self.messages
          .PrivatecaProjectsLocationsCertificateTemplatesCreateRequest(
              parent=parent,
              certificateTemplateId=template_id,
              certificateTemplate=template,
              requestId=request_utils.GenerateRequestId()))
    except api_exceptions.HttpConflictError as e:
      if not overwrite:
        raise ReplicationError(
            location,
            'Certificate template [{}] already exists and the --overwrite flag '
            'was not set.'.format(resource_name))

      return self.client.projects_locations_certificateTemplates.Patch(
          self.messages
          .PrivatecaProjectsLocationsCertificateTemplatesPatchRequest(
              name=resource_name,
              certificateTemplate=template,
              # Always copy all fields. Mask value of '*' doesn't seem to be
              # currently supported by CCFE.
              updateMask='predefined_values,identity_constraints,passthrough_extensions,description,labels',
              requestId=request_utils.GenerateRequestId()))
    except api_exceptions.HttpError as e:
      raise ReplicationError(location, str(e)) from e

  def Run(self, args):
    """Runs the command."""
    self.client = privateca_base.GetClientInstance(api_version='v1')
    self.messages = privateca_base.GetMessagesModule(api_version='v1')

    template_ref = args.CONCEPTS.certificate_template.Parse()
    template = self.client.projects_locations_certificateTemplates.Get(
        self.messages.PrivatecaProjectsLocationsCertificateTemplatesGetRequest(
            name=template_ref.RelativeName()))

    # Name is output-only and will be different for each location.
    template.name = ''

    success_count = 0
    target_locations = args.target_locations
    if args.all_locations:
      target_locations = [
          location for location in locations.GetSupportedLocations('v1')
          if location != template_ref.locationsId
      ]
    for location in target_locations:
      location = location.strip()
      if location == template_ref.locationsId:
        log.warning(
            'Skipping location [{}] since it is the source location.'.format(
                location))
        continue

      try:
        operation = self._CreateOrUpdateTemplate(template_ref.projectsId,
                                                 location, template_ref.Name(),
                                                 template, args.overwrite)
        operations.Await(
            operation,
            'Replicating template to [{}]'.format(location),
            api_version='v1')
        success_count += 1
      except ReplicationError as e:
        if args.continue_on_error:
          log.warning(str(e))
          continue
        raise e

    log.status.Print(
        'Replicated template [{}] to {} out of {} locations.'.format(
            template_ref.RelativeName(), success_count, len(target_locations)))



# -*- coding: utf-8 -*- #
# Copyright 2026 Google LLC. All Rights Reserved.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#    http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
#
# NOTE: This file is autogenerated and should not be edited by hand.
# AUTOGEN_CLI_VERSION: HEAD
"""The super-group for the IAM CLI."""

from googlecloudsdk.calliope import base
from surface.iam import _init_extensions as extensions


@base.ReleaseTracks(base.ReleaseTrack.ALPHA)
@base.Autogenerated
class IamAlpha(extensions.IamAlpha):
  """Manage IAM service accounts and keys.

  The gcloud iam command group lets you manage Google Cloud Identity &
  Access Management (IAM) service accounts and keys.

  Cloud IAM authorizes who can take action on specific resources, giving you
  full control and visibility to manage cloud resources centrally. For
  established enterprises with complex organizational structures, hundreds of
  workgroups and potentially many more projects, Cloud IAM provides a unified
  view into security policy across your entire organization, with built-in
  auditing to ease compliance processes.

  More information on Cloud IAM can be found here:
  https://cloud.google.com/iam and detailed documentation can be found here:
  https://cloud.google.com/iam/docs/.
  """


@base.ReleaseTracks(base.ReleaseTrack.BETA)
@base.Autogenerated
class IamBeta(extensions.IamBeta):
  """Manage IAM service accounts and keys.

  The gcloud iam command group lets you manage Google Cloud Identity &
  Access Management (IAM) service accounts and keys.

  Cloud IAM authorizes who can take action on specific resources, giving you
  full control and visibility to manage cloud resources centrally. For
  established enterprises with complex organizational structures, hundreds of
  workgroups and potentially many more projects, Cloud IAM provides a unified
  view into security policy across your entire organization, with built-in
  auditing to ease compliance processes.

  More information on Cloud IAM can be found here:
  https://cloud.google.com/iam and detailed documentation can be found here:
  https://cloud.google.com/iam/docs/.
  """


@base.ReleaseTracks(base.ReleaseTrack.GA)
@base.Autogenerated
class IamGa(extensions.IamGa):
  """Manage IAM service accounts and keys.

  The gcloud iam command group lets you manage Google Cloud Identity &
  Access Management (IAM) service accounts and keys.

  Cloud IAM authorizes who can take action on specific resources, giving you
  full control and visibility to manage cloud resources centrally. For
  established enterprises with complex organizational structures, hundreds of
  workgroups and potentially many more projects, Cloud IAM provides a unified
  view into security policy across your entire organization, with built-in
  auditing to ease compliance processes.

  More information on Cloud IAM can be found here:
  https://cloud.google.com/iam and detailed documentation can be found here:
  https://cloud.google.com/iam/docs/.
  """



# -*- coding: utf-8 -*- #
# Copyright 2019 Google LLC. All Rights Reserved.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#    http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
"""RemoveIamPolicyBinding command for the Resource Manager - Tag Values CLI."""


import http.client

from googlecloudsdk.api_lib.resource_manager import tags
from googlecloudsdk.api_lib.util import http_retry
from googlecloudsdk.calliope import base
from googlecloudsdk.command_lib.iam import iam_util
from googlecloudsdk.command_lib.resource_manager import tag_arguments as arguments
from googlecloudsdk.command_lib.resource_manager import tag_utils


@base.ReleaseTracks(base.ReleaseTrack.ALPHA, base.ReleaseTrack.BETA,
                    base.ReleaseTrack.GA)
class RemoveIamPolicyBinding(base.Command):
  """Removes a policy binding from the IAM policy of a TagValue.

     Removes an IAM policy binding for a TagValue resource given the binding
     and an identifier for the TagValue. The identifier can be the TagValue's
     namespaced name in the form
     <parent_namespace>/<tagkey_short_name>/<tagvalue_short_name> or the
     TagValue's ID in the form: tagValues/{numeric_id}.
  """

  detailed_help = {
      'EXAMPLES':
          """
          To remove an IAM policy binding for the role of 'roles/editor' for the
          user 'test-user@gmail.com' on the tagValue 'tagValues/111', run:

            $ {command} tagValues/111 --member='user:test-user@gmail.com' --role='roles/editor'

          To remove an IAM policy binding for a TagValue with the name 'dev'
          under 'organization/456/env' for the role of
          'roles/resourcemanager.tagUser' for the user 'test-user@gmail.com',
          run:

            $ {command} 456/env/dev --member='user:test-user@gmail.com' --role='roles/resourcemanager.tagUser'

          See https://cloud.google.com/iam/docs/managing-policies for details of
          policy role and member types.

          """
  }

  @staticmethod
  def Args(parser):
    arguments.AddResourceNameArgToParser(parser)
    iam_util.AddArgsForRemoveIamPolicyBinding(parser, add_condition=True)

  # Allow for retries due to etag-based optimistic concurrency control
  @http_retry.RetryOnHttpStatus(http.client.CONFLICT)
  def Run(self, args):
    service = tags.TagValuesService()
    messages = tags.TagMessages()

    if args.RESOURCE_NAME.find('tagValues/') == 0:
      tag_value = args.RESOURCE_NAME
    else:
      tag_value = tag_utils.GetNamespacedResource(
          args.RESOURCE_NAME, tag_utils.TAG_VALUES
      ).name

    get_iam_policy_req = (
        messages.CloudresourcemanagerTagValuesGetIamPolicyRequest(
            resource=tag_value))
    policy = service.GetIamPolicy(get_iam_policy_req)
    condition = iam_util.ValidateAndExtractConditionMutexRole(args)
    iam_util.RemoveBindingFromIamPolicyWithCondition(policy, args.member,
                                                     args.role, condition,
                                                     args.all)

    set_iam_policy_request = messages.SetIamPolicyRequest(policy=policy)
    request = messages.CloudresourcemanagerTagValuesSetIamPolicyRequest(
        resource=tag_value, setIamPolicyRequest=set_iam_policy_request)
    result = service.SetIamPolicy(request)
    iam_util.LogSetIamPolicy(tag_value, 'TagValue')
    return result



# -*- coding: utf-8 -*- #
# Copyright 2026 Google LLC. All Rights Reserved.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#    http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

"""Implementation of create command for Feature Configs."""

from googlecloudsdk.api_lib.storage import feature_config_api
from googlecloudsdk.calliope import base
from googlecloudsdk.command_lib.storage import flags
from googlecloudsdk.core import properties


@base.Hidden
@base.ReleaseTracks(base.ReleaseTrack.ALPHA)
@base.DefaultUniverseOnly
class Create(base.Command):
  """Create a new Feature Config."""

  hints = base.CommandHint(read_only=False)

  detailed_help = {
      'DESCRIPTION': (
          """
       Create a new Feature Config.
      """
      ),
      'EXAMPLES': (
          """
      To create a Feature Config with config name "my_config" and auto-annotate model "face-detector":

         $ {command} my_config --auto-annotate-models=face-detector
      """
      ),
  }

  @classmethod
  def Args(cls, parser):
    parser.add_argument(
        'CONFIG_ID',
        help='The ID of the feature configuration to create.',
    )
    flags.add_feature_config_description_flag(parser)
    flags.add_feature_config_filter_flags(parser)
    models_group = parser.add_mutually_exclusive_group(required=True)
    flags.add_feature_config_auto_annotate_models_flag(models_group)
    base.ASYNC_FLAG.AddToParser(parser)

  def Run(self, args):
    client = feature_config_api.FeatureConfigApi()
    project = properties.VALUES.core.project.Get(required=True)
    parent = f'projects/{project}/locations/global'

    return client.create_feature_config(
        parent=parent,
        feature_config_id=args.CONFIG_ID,
        description=args.description,
        auto_annotate_models=args.auto_annotate_models,
        include_locations=args.include_locations,
        exclude_locations=args.exclude_locations,
        include_bucket_id_regexes=args.include_bucket_id_regexes,
        exclude_bucket_id_regexes=args.exclude_bucket_id_regexes,
    )



# -*- coding: utf-8 -*- #
# Copyright 2026 Google LLC. All Rights Reserved.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#    http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

"""Network configuration utilities for clusters command group."""

from __future__ import annotations
from typing import Any

from googlecloudsdk.command_lib.cluster_director.clusters import _validator


def _GetNetworkName(args: Any, cluster_ref: Any, network: str) -> str:
  """Returns the full network name including project.

  Args:
    args: The argparse namespace.
    cluster_ref: The cluster resource reference.
    network: The network name.

  Returns:
    The full network name in format
    projects/{project}/global/networks/{network}.
  """

  project = getattr(args, "network_project", None) or (
      cluster_ref.Parent().projectsId
  )
  return f"projects/{project}/global/networks/{network}"


def _GetSubNetworkName(args: Any, cluster_ref: Any, subnetwork: str) -> str:
  """Returns the full subnetwork name including project.

  Args:
    args: The argparse namespace.
    cluster_ref: The cluster resource reference.
    subnetwork: The subnetwork name.

  Returns:
    The full subnetwork name in format projects/{project}/{subnetwork}.
  """
  project = (
      getattr(args, "network_project", None) or cluster_ref.Parent().projectsId
  )
  return f"projects/{project}/{subnetwork}"


def MakeClusterNetworks(
    args: Any, message_module: Any, cluster_ref: Any
) -> Any:
  """Makes a cluster message with network fields.

  Args:
    args: The argparse namespace.
    message_module: The API message module.
    cluster_ref: The cluster resource reference.

  Returns:
    A message_module.Cluster.NetworkResourcesValue object.
  """
  networks = message_module.Cluster.NetworkResourcesValue()
  if args.IsSpecified("create_network"):
    network_id = args.create_network.get("name")
    _validator.ValidateResourceID(network_id)
    network_name = _GetNetworkName(args, cluster_ref, network_id)
    networks.additionalProperties.append(
        message_module.Cluster.NetworkResourcesValue.AdditionalProperty(
            key=f"net-{network_id}",
            value=message_module.NetworkResource(
                config=message_module.NetworkResourceConfig(
                    newNetwork=message_module.NewNetworkConfig(
                        network=network_name,
                        description=args.create_network.get("description"),
                    )
                )
            ),
        )
    )
  if args.IsSpecified("network") and args.IsSpecified("subnet"):
    network_id = args.network
    network_name = _GetNetworkName(args, cluster_ref, network_id)
    networks.additionalProperties.append(
        message_module.Cluster.NetworkResourcesValue.AdditionalProperty(
            key=f"net-{network_id}",
            value=message_module.NetworkResource(
                config=message_module.NetworkResourceConfig(
                    existingNetwork=message_module.ExistingNetworkConfig(
                        network=network_name,
                        subnetwork=_GetSubNetworkName(
                            args, cluster_ref, args.subnet
                        ),
                    )
                )
            ),
        )
    )
  return networks



"""Generated client library for sqladmin version v1beta4."""
# NOTE: This file is autogenerated and should not be edited by hand.

from apitools.base.py import base_api
from googlecloudsdk.generated_clients.apis.sqladmin.v1beta4 import sqladmin_v1beta4_messages as messages


class SqladminV1beta4(base_api.BaseApiClient):
  """Generated client library for service sqladmin version v1beta4."""

  MESSAGES_MODULE = messages
  BASE_URL = 'https://sqladmin.googleapis.com/'
  MTLS_BASE_URL = 'https://sqladmin.mtls.googleapis.com/'

  _PACKAGE = 'sqladmin'
  _SCOPES = ['https://www.googleapis.com/auth/cloud-platform', 'https://www.googleapis.com/auth/sqlservice.admin']
  _VERSION = 'v1beta4'
  _CLIENT_ID = 'CLIENT_ID'
  _CLIENT_SECRET = 'CLIENT_SECRET'
  _USER_AGENT = 'google-cloud-sdk'
  _CLIENT_CLASS_NAME = 'SqladminV1beta4'
  _URL_VERSION = 'v1beta4'
  _API_KEY = None

  def __init__(self, url='', credentials=None,
               get_credentials=True, http=None, model=None,
               log_request=False, log_response=False,
               credentials_args=None, default_global_params=None,
               additional_http_headers=None, response_encoding=None):
    """Create a new sqladmin handle."""
    url = url or self.BASE_URL
    super(SqladminV1beta4, self).__init__(
        url, credentials=credentials,
        get_credentials=get_credentials, http=http, model=model,
        log_request=log_request, log_response=log_response,
        credentials_args=credentials_args,
        default_global_params=default_global_params,
        additional_http_headers=additional_http_headers,
        response_encoding=response_encoding)
    self.backupRuns = self.BackupRunsService(self)
    self.backups = self.BackupsService(self)
    self.blueGreenDeployments = self.BlueGreenDeploymentsService(self)
    self.connect = self.ConnectService(self)
    self.databases = self.DatabasesService(self)
    self.flags = self.FlagsService(self)
    self.instances = self.InstancesService(self)
    self.operations = self.OperationsService(self)
    self.projects_instances = self.ProjectsInstancesService(self)
    self.projects = self.ProjectsService(self)
    self.sslCerts = self.SslCertsService(self)
    self.tiers = self.TiersService(self)
    self.users = self.UsersService(self)

  class BackupRunsService(base_api.BaseApiService):
    """Service class for the backupRuns resource."""

    _NAME = 'backupRuns'

    def __init__(self, client):
      super(SqladminV1beta4.BackupRunsService, self).__init__(client)
      self._upload_configs = {
          }

    def Delete(self, request, global_params=None):
      r"""Deletes the backup taken by a backup run.

      Args:
        request: (SqlBackupRunsDeleteRequest) input message
        global_params: (StandardQueryParameters, default: None) global arguments
      Returns:
        (Operation) The response message.
      """
      config = self.GetMethodConfig('Delete')
      return self._RunMethod(
          config, request, global_params=global_params)

    Delete.method_config = lambda: base_api.ApiMethodInfo(
        http_method='DELETE',
        method_id='sql.backupRuns.delete',
        ordered_params=['project', 'instance', 'id'],
        path_params=['id', 'instance', 'project'],
        query_params=[],
        relative_path='sql/v1beta4/projects/{project}/instances/{instance}/backupRuns/{id}',
        request_field='',
        request_type_name='SqlBackupRunsDeleteRequest',
        response_type_name='Operation',
        supports_download=False,
    )

    def Get(self, request, global_params=None):
      r"""Retrieves a resource containing information about a backup run.

      Args:
        request: (SqlBackupRunsGetRequest) input message
        global_params: (StandardQueryParameters, default: None) global arguments
      Returns:
        (BackupRun) The response message.
      """
      config = self.GetMethodConfig('Get')
      return self._RunMethod(
          config, request, global_params=global_params)

    Get.method_config = lambda: base_api.ApiMethodInfo(
        http_method='GET',
        method_id='sql.backupRuns.get',
        ordered_params=['project', 'instance', 'id'],
        path_params=['id', 'instance', 'project'],
        query_params=[],
        relative_path='sql/v1beta4/projects/{project}/instances/{instance}/backupRuns/{id}',
        request_field='',
        request_type_name='SqlBackupRunsGetRequest',
        response_type_name='BackupRun',
        supports_download=False,
    )

    def Insert(self, request, global_params=None):
      r"""Creates a new backup run on demand.

      Args:
        request: (SqlBackupRunsInsertRequest) input message
        global_params: (StandardQueryParameters, default: None) global arguments
      Returns:
        (Operation) The response message.
      """
      config = self.GetMethodConfig('Insert')
      return self._RunMethod(
          config, request, global_params=global_params)

    Insert.method_config = lambda: base_api.ApiMethodInfo(
        http_method='POST',
        method_id='sql.backupRuns.insert',
        ordered_params=['project', 'instance'],
        path_params=['instance', 'project'],
        query_params=[],
        relative_path='sql/v1beta4/projects/{project}/instances/{instance}/backupRuns',
        request_field='backupRun',
        request_type_name='SqlBackupRunsInsertRequest',
        response_type_name='Operation',
        supports_download=False,
    )

    def List(self, request, global_params=None):
      r"""Lists all backup runs associated with the project or a given instance and configuration in the reverse chronological order of the backup initiation time.

      Args:
        request: (SqlBackupRunsListRequest) input message
        global_params: (StandardQueryParameters, default: None) global arguments
      Returns:
        (BackupRunsListResponse) The response message.
      """
      config = self.GetMethodConfig('List')
      return self._RunMethod(
          config, request, global_params=global_params)

    List.method_config = lambda: base_api.ApiMethodInfo(
        http_method='GET',
        method_id='sql.backupRuns.list',
        ordered_params=['project', 'instance'],
        path_params=['instance', 'project'],
        query_params=['maxResults', 'pageToken'],
        relative_path='sql/v1beta4/projects/{project}/instances/{instance}/backupRuns',
        request_field='',
        request_type_name='SqlBackupRunsListRequest',
        response_type_name='BackupRunsListResponse',
        supports_download=False,
    )

  class BackupsService(base_api.BaseApiService):
    """Service class for the backups resource."""

    _NAME = 'backups'

    def __init__(self, client):
      super(SqladminV1beta4.BackupsService, self).__init__(client)
      self._upload_configs = {
          }

    def CreateBackup(self, request, global_params=None):
      r"""Creates a backup for a Cloud SQL instance. This API can be used only to create on-demand backups.

      Args:
        request: (SqlBackupsCreateBackupRequest) input message
        global_params: (StandardQueryParameters, default: None) global arguments
      Returns:
        (Operation) The response message.
      """
      config = self.GetMethodConfig('CreateBackup')
      return self._RunMethod(
          config, request, global_params=global_params)

    CreateBackup.method_config = lambda: base_api.ApiMethodInfo(
        flat_path='sql/v1beta4/projects/{projectsId}/backups',
        http_method='POST',
        method_id='sql.backups.createBackup',
        ordered_params=['parent'],
        path_params=['parent'],
        query_params=[],
        relative_path='sql/v1beta4/{+parent}/backups',
        request_field='backup',
        request_type_name='SqlBackupsCreateBackupRequest',
        response_type_name='Operation',
        supports_download=False,
    )

    def DeleteBackup(self, request, global_params=None):
      r"""Deletes the backup.

      Args:
        request: (SqlBackupsDeleteBackupRequest) input message
        global_params: (StandardQueryParameters, default: None) global arguments
      Returns:
        (Operation) The response message.
      """
      config = self.GetMethodConfig('DeleteBackup')
      return self._RunMethod(
          config, request, global_params=global_params)

    DeleteBackup.method_config = lambda: base_api.ApiMethodInfo(
        flat_path='sql/v1beta4/projects/{projectsId}/backups/{backupsId}',
        http_method='DELETE',
        method_id='sql.backups.deleteBackup',
        ordered_params=['name'],
        path_params=['name'],
        query_params=[],
        relative_path='sql/v1beta4/{+name}',
        request_field='',
        request_type_name='SqlBackupsDeleteBackupRequest',
        response_type_name='Operation',
        supports_download=False,
    )

    def GetBackup(self, request, global_params=None):
      r"""Retrieves a resource containing information about a backup.

      Args:
        request: (SqlBackupsGetBackupRequest) input message
        global_params: (StandardQueryParameters, default: None) global arguments
      Returns:
        (Backup) The response message.
      """
      config = self.GetMethodConfig('GetBackup')
      return self._RunMethod(
          config, request, global_params=global_params)

    GetBackup.method_config = lambda: base_api.ApiMethodInfo(
        flat_path='sql/v1beta4/projects/{projectsId}/backups/{backupsId}',
        http_method='GET',
        method_id='sql.backups.getBackup',
        ordered_params=['name'],
        path_params=['name'],
        query_params=[],
        relative_path='sql/v1beta4/{+name}',
        request_field='',
        request_type_name='SqlBackupsGetBackupRequest',
        response_type_name='Backup',
        supports_download=False,
    )

    def ListBackups(self, request, global_params=None):
      r"""Lists all backups associated with the project.

      Args:
        request: (SqlBackupsListBackupsRequest) input message
        global_params: (StandardQueryParameters, default: None) global arguments
      Returns:
        (ListBackupsResponse) The response message.
      """
      config = self.GetMethodConfig('ListBackups')
      return self._RunMethod(
          config, request, global_params=global_params)

    ListBackups.method_config = lambda: base_api.ApiMethodInfo(
        flat_path='sql/v1beta4/projects/{projectsId}/backups',
        http_method='GET',
        method_id='sql.backups.listBackups',
        ordered_params=['parent'],
        path_params=['parent'],
        query_params=['filter', 'pageSize', 'pageToken'],
        relative_path='sql/v1beta4/{+parent}/backups',
        request_field='',
        request_type_name='SqlBackupsListBackupsRequest',
        response_type_name='ListBackupsResponse',
        supports_download=False,
    )

    def UpdateBackup(self, request, global_params=None):
      r"""Updates the retention period and the description of the backup. You can use this API to update final backups only.

      Args:
        request: (SqlBackupsUpdateBackupRequest) input message
        global_params: (StandardQueryParameters, default: None) global arguments
      Returns:
        (Operation) The response message.
      """
      config = self.GetMethodConfig('UpdateBackup')
      return self._RunMethod(
          config, request, global_params=global_params)

    UpdateBackup.method_config = lambda: base_api.ApiMethodInfo(
        flat_path='sql/v1beta4/projects/{projectsId}/backups/{backupsId}',
        http_method='PATCH',
        method_id='sql.backups.updateBackup',
        ordered_params=['name'],
        path_params=['name'],
        query_params=['updateMask'],
        relative_path='sql/v1beta4/{+name}',
        request_field='backup',
        request_type_name='SqlBackupsUpdateBackupRequest',
        response_type_name='Operation',
        supports_download=False,
    )

  class BlueGreenDeploymentsService(base_api.BaseApiService):
    """Service class for the blueGreenDeployments resource."""

    _NAME = 'blueGreenDeployments'

    def __init__(self, client):
      super(SqladminV1beta4.BlueGreenDeploymentsService, self).__init__(client)
      self._upload_configs = {
          }

    def Create(self, request, global_params=None):
      r"""Creates a blue-green deployment under a given project and location. This deployment provisions a synchronized green environment (target instance) from a blue production environment (source instance), facilitating updates like major version upgrades on the green instance without impacting the blue instance.

      Args:
        request: (SqlBlueGreenDeploymentsCreateRequest) input message
        global_params: (StandardQueryParameters, default: None) global arguments
      Returns:
        (Operation) The response message.
      """
      config = self.GetMethodConfig('Create')
      return self._RunMethod(
          config, request, global_params=global_params)

    Create.method_config = lambda: base_api.ApiMethodInfo(
        flat_path='sql/v1beta4/projects/{projectsId}/locations/{locationsId}/blueGreenDeployments',
        http_method='POST',
        method_id='sql.blueGreenDeployments.create',
        ordered_params=['parent'],
        path_params=['parent'],
        query_params=['blueGreenDeploymentId'],
        relative_path='sql/v1beta4/{+parent}/blueGreenDeployments',
        request_field='blueGreenDeployment',
        request_type_name='SqlBlueGreenDeploymentsCreateRequest',
        response_type_name='Operation',
        supports_download=False,
    )

    def Delete(self, request, global_params=None):
      r"""Deletes a blue-green deployment, including metadata and underlying resources based on the deployment state. If issued before switchover, this deletes the green instance. If issued after switchover, this deletes the old blue instance (source instance) if the `delete_old_source` field in the request is set to true. All blue-green deployment metadata is permanently deleted. Resources deleted as a result of this operation are no longer accessible and can't be restored.

      Args:
        request: (SqlBlueGreenDeploymentsDeleteRequest) input message
        global_params: (StandardQueryParameters, default: None) global arguments
      Returns:
        (Operation) The response message.
      """
      config = self.GetMethodConfig('Delete')
      return self._RunMethod(
          config, request, global_params=global_params)

    Delete.method_config = lambda: base_api.ApiMethodInfo(
        flat_path='sql/v1beta4/projects/{projectsId}/locations/{locationsId}/blueGreenDeployments/{blueGreenDeploymentsId}',
        http_method='DELETE',
        method_id='sql.blueGreenDeployments.delete',
        ordered_params=['name'],
        path_params=['name'],
        query_params=['deleteOldSource'],
        relative_path='sql/v1beta4/{+name}',
        request_field='',
        request_type_name='SqlBlueGreenDeploymentsDeleteRequest',
        response_type_name='Operation',
        supports_download=False,
    )

    def Get(self, request, global_params=None):
      r"""Retrieves a blue-green deployment resource under a given project and location.

      Args:
        request: (SqlBlueGreenDeploymentsGetRequest) input message
        global_params: (StandardQueryParameters, default: None) global arguments
      Returns:
        (BlueGreenDeployment) The response message.
      """
      config = self.GetMethodConfig('Get')
      return self._RunMethod(
          config, request, global_params=global_params)

    Get.method_config = lambda: base_api.ApiMethodInfo(
        flat_path='sql/v1beta4/projects/{projectsId}/locations/{locationsId}/blueGreenDeployments/{blueGreenDeploymentsId}',
        http_method='GET',
        method_id='sql.blueGreenDeployments.get',
        ordered_params=['name'],
        path_params=['name'],
        query_params=['view'],
        relative_path='sql/v1beta4/{+name}',
        request_field='',
        request_type_name='SqlBlueGreenDeploymentsGetRequest',
        response_type_name='BlueGreenDeployment',
        supports_download=False,
    )

    def List(self, request, global_params=None):
      r"""Lists blue-green deployments under a given project.

      Args:
        request: (SqlBlueGreenDeploymentsListRequest) input message
        global_params: (StandardQueryParameters, default: None) global arguments
      Returns:
        (ListBlueGreenDeploymentsResponse) The response message.
      """
      config = self.GetMethodConfig('List')
      return self._RunMethod(
          config, request, global_params=global_params)

    List.method_config = lambda: base_api.ApiMethodInfo(
        flat_path='sql/v1beta4/projects/{projectsId}/locations/{locationsId}/blueGreenDeployments',
        http_method='GET',
        method_id='sql.blueGreenDeployments.list',
        ordered_params=['parent'],
        path_params=['parent'],
        query_params=['filter', 'orderBy', 'pageSize', 'pageToken'],
        relative_path='sql/v1beta4/{+parent}/blueGreenDeployments',
        request_field='',
        request_type_name='SqlBlueGreenDeploymentsListRequest',
        response_type_name='ListBlueGreenDeploymentsResponse',
        supports_download=False,
    )

    def Switchover(self, request, global_params=None):
      r"""Switches over to green instance for a blue-green deployment.

      Args:
        request: (SqlBlueGreenDeploymentsSwitchoverRequest) input message
        global_params: (StandardQueryParameters, default: None) global arguments
      Returns:
        (Operation) The response message.
      """
      config = self.GetMethodConfig('Switchover')
      return self._RunMethod(
          config, request, global_params=global_params)

    Switchover.method_config = lambda: base_api.ApiMethodInfo(
        flat_path='sql/v1beta4/projects/{projectsId}/locations/{locationsId}/blueGreenDeployments/{blueGreenDeploymentsId}:switchover',
        http_method='POST',
        method_id='sql.blueGreenDeployments.switchover',
        ordered_params=['name'],
        path_params=['name'],
        query_params=[],
        relative_path='sql/v1beta4/{+name}:switchover',
        request_field='switchoverBlueGreenDeploymentRequest',
        request_type_name='SqlBlueGreenDeploymentsSwitchoverRequest',
        response_type_name='Operation',
        supports_download=False,
    )

  class ConnectService(base_api.BaseApiService):
    """Service class for the connect resource."""

    _NAME = 'connect'

    def __init__(self, client):
      super(SqladminV1beta4.ConnectService, self).__init__(client)
      self._upload_configs = {
          }

    def GenerateEphemeralCert(self, request, global_params=None):
      r"""Generates a short-lived X509 certificate containing the provided public key and signed by a private key specific to the target instance. Users may use the certificate to authenticate as themselves when connecting to the database.

      Args:
        request: (SqlConnectGenerateEphemeralRequest) input message
        global_params: (StandardQueryParameters, default: None) global arguments
      Returns:
        (GenerateEphemeralCertResponse) The response message.
      """
      config = self.GetMethodConfig('GenerateEphemeralCert')
      return self._RunMethod(
          config, request, global_params=global_params)

    GenerateEphemeralCert.method_config = lambda: base_api.ApiMethodInfo(
        http_method='POST',
        method_id='sql.connect.generateEphemeral',
        ordered_params=['project', 'instance'],
        path_params=['instance', 'project'],
        query_params=[],
        relative_path='sql/v1beta4/projects/{project}/instances/{instance}:generateEphemeralCert',
        request_field='generateEphemeralCertRequest',
        request_type_name='SqlConnectGenerateEphemeralRequest',
        response_type_name='GenerateEphemeralCertResponse',
        supports_download=False,
    )

    def Get(self, request, global_params=None):
      r"""Retrieves connect settings about a Cloud SQL instance.

      Args:
        request: (SqlConnectGetRequest) input message
        global_params: (StandardQueryParameters, default: None) global arguments
      Returns:
        (ConnectSettings) The response message.
      """
      config = self.GetMethodConfig('Get')
      return self._RunMethod(
          config, request, global_params=global_params)

    Get.method_config = lambda: base_api.ApiMethodInfo(
        http_method='GET',
        method_id='sql.connect.get',
        ordered_params=['project', 'instance'],
        path_params=['instance', 'project'],
        query_params=['readTime'],
        relative_path='sql/v1beta4/projects/{project}/instances/{instance}/connectSettings',
        request_field='',
        request_type_name='SqlConnectGetRequest',
        response_type_name='ConnectSettings',
        supports_download=False,
    )

    def Resolve(self, request, global_params=None):
      r"""Retrieves connect settings about a Cloud SQL instance using the instance DNS name.

      Args:
        request: (SqlConnectResolveRequest) input message
        global_params: (StandardQueryParameters, default: None) global arguments

      Returns:
        (ConnectSettings) The response message.
      """
      config = self.GetMethodConfig('Resolve')
      return self._RunMethod(config, request, global_params=global_params)

    Resolve.method_config = lambda: base_api.ApiMethodInfo(
        http_method='GET',
        method_id='sql.connect.resolve',
        ordered_params=['location', 'dnsName'],
        path_params=['dnsName', 'location'],
        query_params=[],
        relative_path='sql/v1beta4/locations/{location}/dns/{dnsName}:resolveConnectSettings',
        request_field='',
        request_type_name='SqlConnectResolveRequest',
        response_type_name='ConnectSettings',
        supports_download=False,
    )

  class DatabasesService(base_api.BaseApiService):
    """Service class for the databases resource."""

    _NAME = 'databases'

    def __init__(self, client):
      super(SqladminV1beta4.DatabasesService, self).__init__(client)
      self._upload_configs = {
          }

    def Delete(self, request, global_params=None):
      r"""Deletes a database from a Cloud SQL instance.

      Args:
        request: (SqlDatabasesDeleteRequest) input message
        global_params: (StandardQueryParameters, default: None) global arguments
      Returns:
        (Operation) The response message.
      """
      config = self.GetMethodConfig('Delete')
      return self._RunMethod(
          config, request, global_params=global_params)

    Delete.method_config = lambda: base_api.ApiMethodInfo(
        http_method='DELETE',
        method_id='sql.databases.delete',
        ordered_params=['project', 'instance', 'database'],
        path_params=['database', 'instance', 'project'],
        query_params=[],
        relative_path='sql/v1beta4/projects/{project}/instances/{instance}/databases/{database}',
        request_field='',
        request_type_name='SqlDatabasesDeleteRequest',
        response_type_name='Operation',
        supports_download=False,
    )

    def Get(self, request, global_params=None):
      r"""Retrieves a resource containing information about a database inside a Cloud SQL instance.

      Args:
        request: (SqlDatabasesGetRequest) input message
        global_params: (StandardQueryParameters, default: None) global arguments
      Returns:
        (Database) The response message.
      """
      config = self.GetMethodConfig('Get')
      return self._RunMethod(
          config, request, global_params=global_params)

    Get.method_config = lambda: base_api.ApiMethodInfo(
        http_method='GET',
        method_id='sql.databases.get',
        ordered_params=['project', 'instance', 'database'],
        path_params=['database', 'instance', 'project'],
        query_params=[],
        relative_path='sql/v1beta4/projects/{project}/instances/{instance}/databases/{database}',
        request_field='',
        request_type_name='SqlDatabasesGetRequest',
        response_type_name='Database',
        supports_download=False,
    )

    def Insert(self, request, global_params=None):
      r"""Inserts a resource containing information about a database inside a Cloud SQL instance. **Note:** You can't modify the default character set and collation.

      Args:
        request: (Database) input message
        global_params: (StandardQueryParameters, default: None) global arguments
      Returns:
        (Operation) The response message.
      """
      config = self.GetMethodConfig('Insert')
      return self._RunMethod(
          config, request, global_params=global_params)

    Insert.method_config = lambda: base_api.ApiMethodInfo(
        http_method='POST',
        method_id='sql.databases.insert',
        ordered_params=['project', 'instance'],
        path_params=['instance', 'project'],
        query_params=[],
        relative_path='sql/v1beta4/projects/{project}/instances/{instance}/databases',
        request_field='<request>',
        request_type_name='Database',
        response_type_name='Operation',
        supports_download=False,
    )

    def List(self, request, global_params=None):
      r"""Lists databases in the specified Cloud SQL instance.

      Args:
        request: (SqlDatabasesListRequest) input message
        global_params: (StandardQueryParameters, default: None) global arguments
      Returns:
        (DatabasesListResponse) The response message.
      """
      config = self.GetMethodConfig('List')
      return self._RunMethod(
          config, request, global_params=global_params)

    List.method_config = lambda: base_api.ApiMethodInfo(
        http_method='GET',
        method_id='sql.databases.list',
        ordered_params=['project', 'instance'],
        path_params=['instance', 'project'],
        query_params=[],
        relative_path='sql/v1beta4/projects/{project}/instances/{instance}/databases',
        request_field='',
        request_type_name='SqlDatabasesListRequest',
        response_type_name='DatabasesListResponse',
        supports_download=False,
    )

    def Patch(self, request, global_params=None):
      r"""Partially updates a resource containing information about a database inside a Cloud SQL instance. This method supports patch semantics.

      Args:
        request: (SqlDatabasesPatchRequest) input message
        global_params: (StandardQueryParameters, default: None) global arguments
      Returns:
        (Operation) The response message.
      """
      config = self.GetMethodConfig('Patch')
      return self._RunMethod(
          config, request, global_params=global_params)

    Patch.method_config = lambda: base_api.ApiMethodInfo(
        http_method='PATCH',
        method_id='sql.databases.patch',
        ordered_params=['project', 'instance', 'database'],
        path_params=['database', 'instance', 'project'],
        query_params=[],
        relative_path='sql/v1beta4/projects/{project}/instances/{instance}/databases/{database}',
        request_field='databaseResource',
        request_type_name='SqlDatabasesPatchRequest',
        response_type_name='Operation',
        supports_download=False,
    )

    def Update(self, request, global_params=None):
      r"""Updates a resource containing information about a database inside a Cloud SQL instance.

      Args:
        request: (SqlDatabasesUpdateRequest) input message
        global_params: (StandardQueryParameters, default: None) global arguments
      Returns:
        (Operation) The response message.
      """
      config = self.GetMethodConfig('Update')
      return self._RunMethod(
          config, request, global_params=global_params)

    Update.method_config = lambda: base_api.ApiMethodInfo(
        http_method='PUT',
        method_id='sql.databases.update',
        ordered_params=['project', 'instance', 'database'],
        path_params=['database', 'instance', 'project'],
        query_params=[],
        relative_path='sql/v1beta4/projects/{project}/instances/{instance}/databases/{database}',
        request_field='databaseResource',
        request_type_name='SqlDatabasesUpdateRequest',
        response_type_name='Operation',
        supports_download=False,
    )

  class FlagsService(base_api.BaseApiService):
    """Service class for the flags resource."""

    _NAME = 'flags'

    def __init__(self, client):
      super(SqladminV1beta4.FlagsService, self).__init__(client)
      self._upload_configs = {
          }

    def List(self, request, global_params=None):
      r"""Lists all available database flags for Cloud SQL instances.

      Args:
        request: (SqlFlagsListRequest) input message
        global_params: (StandardQueryParameters, default: None) global arguments
      Returns:
        (FlagsListResponse) The response message.
      """
      config = self.GetMethodConfig('List')
      return self._RunMethod(
          config, request, global_params=global_params)

    List.method_config = lambda: base_api.ApiMethodInfo(
        http_method='GET',
        method_id='sql.flags.list',
        ordered_params=[],
        path_params=[],
        query_params=['databaseVersion', 'flagScope'],
        relative_path='sql/v1beta4/flags',
        request_field='',
        request_type_name='SqlFlagsListRequest',
        response_type_name='FlagsListResponse',
        supports_download=False,
    )

  class InstancesService(base_api.BaseApiService):
    """Service class for the instances resource."""

    _NAME = 'instances'

    def __init__(self, client):
      super(SqladminV1beta4.InstancesService, self).__init__(client)
      self._upload_configs = {
          }

    def ListEntraIdCertificates(self, request, global_params=None):
      r"""Lists all versions of EntraID certificates for the specified instance. There can be up to three sets of certificates listed: the certificate that is currently in use, a future that has been added but not yet used to sign a certificate, and a certificate that has been rotated out.

      Args:
        request: (SqlInstancesListEntraIdCertificatesRequest) input message
        global_params: (StandardQueryParameters, default: None) global arguments
      Returns:
        (InstancesListEntraIdCertificatesResponse) The response message.
      """
      config = self.GetMethodConfig('ListEntraIdCertificates')
      return self._RunMethod(
          config, request, global_params=global_params)

    ListEntraIdCertificates.method_config = lambda: base_api.ApiMethodInfo(
        http_method='GET',
        method_id='sql.instances.ListEntraIdCertificates',
        ordered_params=['project', 'instance'],
        path_params=['instance', 'project'],
        query_params=[],
        relative_path='sql/v1beta4/projects/{project}/instances/{instance}/listEntraIdCertificates',
        request_field='',
        request_type_name='SqlInstancesListEntraIdCertificatesRequest',
        response_type_name='InstancesListEntraIdCertificatesResponse',
        supports_download=False,
    )

    def ListServerCertificates(self, request, global_params=None):
      r"""Lists all versions of server certificates and certificate authorities (CAs) for the specified instance. There can be up to three sets of certs listed: the certificate that is currently in use, a future that has been added but not yet used to sign a certificate, and a certificate that has been rotated out. For instances not using Certificate Authority Service (CAS) server CA, use ListServerCas instead.

      Args:
        request: (SqlInstancesListServerCertificatesRequest) input message
        global_params: (StandardQueryParameters, default: None) global arguments
      Returns:
        (InstancesListServerCertificatesResponse) The response message.
      """
      config = self.GetMethodConfig('ListServerCertificates')
      return self._RunMethod(
          config, request, global_params=global_params)

    ListServerCertificates.method_config = lambda: base_api.ApiMethodInfo(
        http_method='GET',
        method_id='sql.instances.ListServerCertificates',
        ordered_params=['project', 'instance'],
        path_params=['instance', 'project'],
        query_params=[],
        relative_path='sql/v1beta4/projects/{project}/instances/{instance}/listServerCertificates',
        request_field='',
        request_type_name='SqlInstancesListServerCertificatesRequest',
        response_type_name='InstancesListServerCertificatesResponse',
        supports_download=False,
    )

    def RotateEntraIdCertificate(self, request, global_params=None):
      r"""Rotates the Entra Id certificate version to one previously added with the addEntraIdCertificate method.

      Args:
        request: (SqlInstancesRotateEntraIdCertificateRequest) input message
        global_params: (StandardQueryParameters, default: None) global arguments
      Returns:
        (Operation) The response message.
      """
      config = self.GetMethodConfig('RotateEntraIdCertificate')
      return self._RunMethod(
          config, request, global_params=global_params)

    RotateEntraIdCertificate.method_config = lambda: base_api.ApiMethodInfo(
        http_method='POST',
        method_id='sql.instances.RotateEntraIdCertificate',
        ordered_params=['project', 'instance'],
        path_params=['instance', 'project'],
        query_params=[],
        relative_path='sql/v1beta4/projects/{project}/instances/{instance}/rotateEntraIdCertificate',
        request_field='instancesRotateEntraIdCertificateRequest',
        request_type_name='SqlInstancesRotateEntraIdCertificateRequest',
        response_type_name='Operation',
        supports_download=False,
    )

    def RotateServerCertificate(self, request, global_params=None):
      r"""Rotates the server certificate version to one previously added with the addServerCertificate method. For instances not using Certificate Authority Service (CAS) server CA, use RotateServerCa instead.

      Args:
        request: (SqlInstancesRotateServerCertificateRequest) input message
        global_params: (StandardQueryParameters, default: None) global arguments
      Returns:
        (Operation) The response message.
      """
      config = self.GetMethodConfig('RotateServerCertificate')
      return self._RunMethod(
          config, request, global_params=global_params)

    RotateServerCertificate.method_config = lambda: base_api.ApiMethodInfo(
        http_method='POST',
        method_id='sql.instances.RotateServerCertificate',
        ordered_params=['project', 'instance'],
        path_params=['instance', 'project'],
        query_params=[],
        relative_path='sql/v1beta4/projects/{project}/instances/{instance}/rotateServerCertificate',
        request_field='instancesRotateServerCertificateRequest',
        request_type_name='SqlInstancesRotateServerCertificateRequest',
        response_type_name='Operation',
        supports_download=False,
    )

    def AcquireSsrsLease(self, request, global_params=None):
      r"""Acquire a lease for the setup of SQL Server Reporting Services (SSRS).

      Args:
        request: (SqlInstancesAcquireSsrsLeaseRequest) input message
        global_params: (StandardQueryParameters, default: None) global arguments
      Returns:
        (SqlInstancesAcquireSsrsLeaseResponse) The response message.
      """
      config = self.GetMethodConfig('AcquireSsrsLease')
      return self._RunMethod(
          config, request, global_params=global_params)

    AcquireSsrsLease.method_config = lambda: base_api.ApiMethodInfo(
        http_method='POST',
        method_id='sql.instances.acquireSsrsLease',
        ordered_params=['project', 'instance'],
        path_params=['instance', 'project'],
        query_params=[],
        relative_path='sql/v1beta4/projects/{project}/instances/{instance}/acquireSsrsLease',
        request_field='instancesAcquireSsrsLeaseRequest',
        request_type_name='SqlInstancesAcquireSsrsLeaseRequest',
        response_type_name='SqlInstancesAcquireSsrsLeaseResponse',
        supports_download=False,
    )

    def AddEntraIdCertificate(self, request, global_params=None):
      r"""Adds a new Entra ID certificate for the specified instance. If an Entra ID certificate was previously added but never used in a certificate rotation, this operation replaces that version.

      Args:
        request: (SqlInstancesAddEntraIdCertificateRequest) input message
        global_params: (StandardQueryParameters, default: None) global arguments
      Returns:
        (Operation) The response message.
      """
      config = self.GetMethodConfig('AddEntraIdCertificate')
      return self._RunMethod(
          config, request, global_params=global_params)

    AddEntraIdCertificate.method_config = lambda: base_api.ApiMethodInfo(
        http_method='POST',
        method_id='sql.instances.addEntraIdCertificate',
        ordered_params=['project', 'instance'],
        path_params=['instance', 'project'],
        query_params=[],
        relative_path='sql/v1beta4/projects/{project}/instances/{instance}/addEntraIdCertificate',
        request_field='',
        request_type_name='SqlInstancesAddEntraIdCertificateRequest',
        response_type_name='Operation',
        supports_download=False,
    )

    def AddReplicationSource(self, request, global_params=None):
      r"""Add source instance names to replicate from. Only applicable to external server writable destinations.

      Args:
        request: (SqlInstancesAddReplicationSourceRequest) input message
        global_params: (StandardQueryParameters, default: None) global arguments
      Returns:
        (Operation) The response message.
      """
      config = self.GetMethodConfig('AddReplicationSource')
      return self._RunMethod(
          config, request, global_params=global_params)

    AddReplicationSource.method_config = lambda: base_api.ApiMethodInfo(
        http_method='POST',
        method_id='sql.instances.addReplicationSource',
        ordered_params=['project', 'instance'],
        path_params=['instance', 'project'],
        query_params=[],
        relative_path='sql/v1beta4/projects/{project}/instances/{instance}/addReplicationSource',
        request_field='sqlInstancesAddReplicationSourceRequest',
        request_type_name='SqlInstancesAddReplicationSourceRequest',
        response_type_name='Operation',
        supports_download=False,
    )

    def AddServerCa(self, request, global_params=None):
      r"""Add a new trusted Certificate Authority (CA) version for the specified instance. Required to prepare for a certificate rotation. If a CA version was previously added but never used in a certificate rotation, this operation replaces that version. There cannot be more than one CA version waiting to be rotated in. For instances that have enabled Certificate Authority Service (CAS) based server CA, use AddServerCertificate to add a new server certificate.

      Args:
        request: (SqlInstancesAddServerCaRequest) input message
        global_params: (StandardQueryParameters, default: None) global arguments
      Returns:
        (Operation) The response message.
      """
      config = self.GetMethodConfig('AddServerCa')
      return self._RunMethod(
          config, request, global_params=global_params)

    AddServerCa.method_config = lambda: base_api.ApiMethodInfo(
        http_method='POST',
        method_id='sql.instances.addServerCa',
        ordered_params=['project', 'instance'],
        path_params=['instance', 'project'],
        query_params=[],
        relative_path='sql/v1beta4/projects/{project}/instances/{instance}/addServerCa',
        request_field='',
        request_type_name='SqlInstancesAddServerCaRequest',
        response_type_name='Operation',
        supports_download=False,
    )

    def AddServerCertificate(self, request, global_params=None):
      r"""Add a new trusted server certificate version for the specified instance using Certificate Authority Service (CAS) server CA. Required to prepare for a certificate rotation. If a server certificate version was previously added but never used in a certificate rotation, this operation replaces that version. There cannot be more than one certificate version waiting to be rotated in. For instances not using CAS server CA, use AddServerCa instead.

      Args:
        request: (SqlInstancesAddServerCertificateRequest) input message
        global_params: (StandardQueryParameters, default: None) global arguments
      Returns:
        (Operation) The response message.
      """
      config = self.GetMethodConfig('AddServerCertificate')
      return self._RunMethod(
          config, request, global_params=global_params)

    AddServerCertificate.method_config = lambda: base_api.ApiMethodInfo(
        http_method='POST',
        method_id='sql.instances.addServerCertificate',
        ordered_params=['project', 'instance'],
        path_params=['instance', 'project'],
        query_params=[],
        relative_path='sql/v1beta4/projects/{project}/instances/{instance}/addServerCertificate',
        request_field='',
        request_type_name='SqlInstancesAddServerCertificateRequest',
        response_type_name='Operation',
        supports_download=False,
    )

    def Clone(self, request, global_params=None):
      r"""Creates a Cloud SQL instance as a clone of the source instance. Using this operation might cause your instance to restart.

      Args:
        request: (SqlInstancesCloneRequest) input message
        global_params: (StandardQueryParameters, default: None) global arguments
      Returns:
        (Operation) The response message.
      """
      config = self.GetMethodConfig('Clone')
      return self._RunMethod(
          config, request, global_params=global_params)

    Clone.method_config = lambda: base_api.ApiMethodInfo(
        http_method='POST',
        method_id='sql.instances.clone',
        ordered_params=['project', 'instance'],
        path_params=['instance', 'project'],
        query_params=[],
        relative_path='sql/v1beta4/projects/{project}/instances/{instance}/clone',
        request_field='instancesCloneRequest',
        request_type_name='SqlInstancesCloneRequest',
        response_type_name='Operation',
        supports_download=False,
    )

    def Delete(self, request, global_params=None):
      r"""Deletes a Cloud SQL instance.

      Args:
        request: (SqlInstancesDeleteRequest) input message
        global_params: (StandardQueryParameters, default: None) global arguments
      Returns:
        (Operation) The response message.
      """
      config = self.GetMethodConfig('Delete')
      return self._RunMethod(
          config, request, global_params=global_params)

    Delete.method_config = lambda: base_api.ApiMethodInfo(
        http_method='DELETE',
        method_id='sql.instances.delete',
        ordered_params=['project', 'instance'],
        path_params=['instance', 'project'],
        query_params=['enableFinalBackup', 'finalBackupDescription', 'finalBackupExpiryTime', 'finalBackupTtlDays', 'retainBackups', 'retainBackupsExpiryTime', 'retainBackupsTtlDays', 'skipFinalBackup'],
        relative_path='sql/v1beta4/projects/{project}/instances/{instance}',
        request_field='',
        request_type_name='SqlInstancesDeleteRequest',
        response_type_name='Operation',
        supports_download=False,
    )

    def Demote(self, request, global_params=None):
      r"""Demotes an existing standalone instance to be a Cloud SQL read replica for an external database server.

      Args:
        request: (SqlInstancesDemoteRequest) input message
        global_params: (StandardQueryParameters, default: None) global arguments
      Returns:
        (Operation) The response message.
      """
      config = self.GetMethodConfig('Demote')
      return self._RunMethod(
          config, request, global_params=global_params)

    Demote.method_config = lambda: base_api.ApiMethodInfo(
        http_method='POST',
        method_id='sql.instances.demote',
        ordered_params=['project', 'instance'],
        path_params=['instance', 'project'],
        query_params=[],
        relative_path='sql/v1beta4/projects/{project}/instances/{instance}/demote',
        request_field='instancesDemoteRequest',
        request_type_name='SqlInstancesDemoteRequest',
        response_type_name='Operation',
        supports_download=False,
    )

    def DemoteMaster(self, request, global_params=None):
      r"""Demotes the stand-alone instance to be a Cloud SQL read replica for an external database server.

      Args:
        request: (SqlInstancesDemoteMasterRequest) input message
        global_params: (StandardQueryParameters, default: None) global arguments
      Returns:
        (Operation) The response message.
      """
      config = self.GetMethodConfig('DemoteMaster')
      return self._RunMethod(
          config, request, global_params=global_params)

    DemoteMaster.method_config = lambda: base_api.ApiMethodInfo(
        http_method='POST',
        method_id='sql.instances.demoteMaster',
        ordered_params=['project', 'instance'],
        path_params=['instance', 'project'],
        query_params=[],
        relative_path='sql/v1beta4/projects/{project}/instances/{instance}/demoteMaster',
        request_field='instancesDemoteMasterRequest',
        request_type_name='SqlInstancesDemoteMasterRequest',
        response_type_name='Operation',
        supports_download=False,
    )

    def ExecuteSql(self, request, global_params=None):
      r"""Execute SQL statements.

      Args:
        request: (SqlInstancesExecuteSqlRequest) input message
        global_params: (StandardQueryParameters, default: None) global arguments
      Returns:
        (SqlInstancesExecuteSqlResponse) The response message.
      """
      config = self.GetMethodConfig('ExecuteSql')
      return self._RunMethod(
          config, request, global_params=global_params)

    ExecuteSql.method_config = lambda: base_api.ApiMethodInfo(
        http_method='POST',
        method_id='sql.instances.executeSql',
        ordered_params=['project', 'instance'],
        path_params=['instance', 'project'],
        query_params=[],
        relative_path='sql/v1beta4/projects/{project}/instances/{instance}/executeSql',
        request_field='executeSqlPayload',
        request_type_name='SqlInstancesExecuteSqlRequest',
        response_type_name='SqlInstancesExecuteSqlResponse',
        supports_download=False,
    )

    def Export(self, request, global_params=None):
      r"""Exports data from a Cloud SQL instance to a Cloud Storage bucket as a SQL dump or CSV file.

      Args:
        request: (SqlInstancesExportRequest) input message
        global_params: (StandardQueryParameters, default: None) global arguments
      Returns:
        (Operation) The response message.
      """
      config = self.GetMethodConfig('Export')
      return self._RunMethod(
          config, request, global_params=global_params)

    Export.method_config = lambda: base_api.ApiMethodInfo(
        http_method='POST',
        method_id='sql.instances.export',
        ordered_params=['project', 'instance'],
        path_params=['instance', 'project'],
        query_params=[],
        relative_path='sql/v1beta4/projects/{project}/instances/{instance}/export',
        request_field='instancesExportRequest',
        request_type_name='SqlInstancesExportRequest',
        response_type_name='Operation',
        supports_download=False,
    )

    def Failover(self, request, global_params=None):
      r"""Initiates a manual failover of a high availability (HA) primary instance to a standby instance, which becomes the primary instance. Users are then rerouted to the new primary. For more information, see the [Overview of high availability](https://cloud.google.com/sql/docs/mysql/high-availability) page in the Cloud SQL documentation. If using Legacy HA (MySQL only), this causes the instance to failover to its failover replica instance.

      Args:
        request: (SqlInstancesFailoverRequest) input message
        global_params: (StandardQueryParameters, default: None) global arguments
      Returns:
        (Operation) The response message.
      """
      config = self.GetMethodConfig('Failover')
      return self._RunMethod(
          config, request, global_params=global_params)

    Failover.method_config = lambda: base_api.ApiMethodInfo(
        http_method='POST',
        method_id='sql.instances.failover',
        ordered_params=['project', 'instance'],
        path_params=['instance', 'project'],
        query_params=[],
        relative_path='sql/v1beta4/projects/{project}/instances/{instance}/failover',
        request_field='instancesFailoverRequest',
        request_type_name='SqlInstancesFailoverRequest',
        response_type_name='Operation',
        supports_download=False,
    )

    def Get(self, request, global_params=None):
      r"""Retrieves a resource containing information about a Cloud SQL instance.

      Args:
        request: (SqlInstancesGetRequest) input message
        global_params: (StandardQueryParameters, default: None) global arguments
      Returns:
        (DatabaseInstance) The response message.
      """
      config = self.GetMethodConfig('Get')
      return self._RunMethod(
          config, request, global_params=global_params)

    Get.method_config = lambda: base_api.ApiMethodInfo(
        http_method='GET',
        method_id='sql.instances.get',
        ordered_params=['project', 'instance'],
        path_params=['instance', 'project'],
        query_params=[],
        relative_path='sql/v1beta4/projects/{project}/instances/{instance}',
        request_field='',
        request_type_name='SqlInstancesGetRequest',
        response_type_name='DatabaseInstance',
        supports_download=False,
    )

    def Import(self, request, global_params=None):
      r"""Imports data into a Cloud SQL instance from a SQL dump or CSV file in Cloud Storage.

      Args:
        request: (SqlInstancesImportRequest) input message
        global_params: (StandardQueryParameters, default: None) global arguments
      Returns:
        (Operation) The response message.
      """
      config = self.GetMethodConfig('Import')
      return self._RunMethod(
          config, request, global_params=global_params)

    Import.method_config = lambda: base_api.ApiMethodInfo(
        http_method='POST',
        method_id='sql.instances.import',
        ordered_params=['project', 'instance'],
        path_params=['instance', 'project'],
        query_params=[],
        relative_path='sql/v1beta4/projects/{project}/instances/{instance}/import',
        request_field='instancesImportRequest',
        request_type_name='SqlInstancesImportRequest',
        response_type_name='Operation',
        supports_download=False,
    )

    def Insert(self, request, global_params=None):
      r"""Creates a new Cloud SQL instance.

      Args:
        request: (SqlInstancesInsertRequest) input message
        global_params: (StandardQueryParameters, default: None) global arguments
      Returns:
        (Operation) The response message.
      """
      config = self.GetMethodConfig('Insert')
      return self._RunMethod(
          config, request, global_params=global_params)

    Insert.method_config = lambda: base_api.ApiMethodInfo(
        http_method='POST',
        method_id='sql.instances.insert',
        ordered_params=['project'],
        path_params=['project'],
        query_params=[],
        relative_path='sql/v1beta4/projects/{project}/instances',
        request_field='databaseInstance',
        request_type_name='SqlInstancesInsertRequest',
        response_type_name='Operation',
        supports_download=False,
    )

    def List(self, request, global_params=None):
      r"""Lists instances under a given project.

      Args:
        request: (SqlInstancesListRequest) input message
        global_params: (StandardQueryParameters, default: None) global arguments
      Returns:
        (InstancesListResponse) The response message.
      """
      config = self.GetMethodConfig('List')
      return self._RunMethod(
          config, request, global_params=global_params)

    List.method_config = lambda: base_api.ApiMethodInfo(
        http_method='GET',
        method_id='sql.instances.list',
        ordered_params=['project'],
        path_params=['project'],
        query_params=['filter', 'maxResults', 'pageToken'],
        relative_path='sql/v1beta4/projects/{project}/instances',
        request_field='',
        request_type_name='SqlInstancesListRequest',
        response_type_name='InstancesListResponse',
        supports_download=False,
    )

    def ListServerCas(self, request, global_params=None):
      r"""Lists all of the trusted Certificate Authorities (CAs) for the specified instance. There can be up to three CAs listed: the CA that was used to sign the certificate that is currently in use, a CA that has been added but not yet used to sign a certificate, and a CA used to sign a certificate that has previously rotated out.

      Args:
        request: (SqlInstancesListServerCasRequest) input message
        global_params: (StandardQueryParameters, default: None) global arguments
      Returns:
        (InstancesListServerCasResponse) The response message.
      """
      config = self.GetMethodConfig('ListServerCas')
      return self._RunMethod(
          config, request, global_params=global_params)

    ListServerCas.method_config = lambda: base_api.ApiMethodInfo(
        http_method='GET',
        method_id='sql.instances.listServerCas',
        ordered_params=['project', 'instance'],
        path_params=['instance', 'project'],
        query_params=[],
        relative_path='sql/v1beta4/projects/{project}/instances/{instance}/listServerCas',
        request_field='',
        request_type_name='SqlInstancesListServerCasRequest',
        response_type_name='InstancesListServerCasResponse',
        supports_download=False,
    )

    def Patch(self, request, global_params=None):
      r"""Partially updates settings of a Cloud SQL instance by merging the request with the current configuration. This method supports patch semantics.

      Args:
        request: (SqlInstancesPatchRequest) input message
        global_params: (StandardQueryParameters, default: None) global arguments
      Returns:
        (Operation) The response message.
      """
      config = self.GetMethodConfig('Patch')
      return self._RunMethod(
          config, request, global_params=global_params)

    Patch.method_config = lambda: base_api.ApiMethodInfo(
        http_method='PATCH',
        method_id='sql.instances.patch',
        ordered_params=['project', 'instance'],
        path_params=['instance', 'project'],
        query_params=['enforcePsaWriteEndpoint'],
        relative_path='sql/v1beta4/projects/{project}/instances/{instance}',
        request_field='databaseInstance',
        request_type_name='SqlInstancesPatchRequest',
        response_type_name='Operation',
        supports_download=False,
    )

    def PointInTimeRestore(self, request, global_params=None):
      r"""Point in time restore for an instance managed by Google Cloud Backup and Disaster Recovery.

      Args:
        request: (SqlInstancesPointInTimeRestoreRequest) input message
        global_params: (StandardQueryParameters, default: None) global arguments
      Returns:
        (Operation) The response message.
      """
      config = self.GetMethodConfig('PointInTimeRestore')
      return self._RunMethod(
          config, request, global_params=global_params)

    PointInTimeRestore.method_config = lambda: base_api.ApiMethodInfo(
        flat_path='sql/v1beta4/projects/{projectsId}:pointInTimeRestore',
        http_method='POST',
        method_id='sql.instances.pointInTimeRestore',
        ordered_params=['parent'],
        path_params=['parent'],
        query_params=[],
        relative_path='sql/v1beta4/{+parent}:pointInTimeRestore',
        request_field='pointInTimeRestoreContext',
        request_type_name='SqlInstancesPointInTimeRestoreRequest',
        response_type_name='Operation',
        supports_download=False,
    )

    def PreCheckMajorVersionUpgrade(self, request, global_params=None):
      r"""Execute MVU Pre-checks.

      Args:
        request: (SqlInstancesPreCheckMajorVersionUpgradeRequest) input message
        global_params: (StandardQueryParameters, default: None) global arguments
      Returns:
        (Operation) The response message.
      """
      config = self.GetMethodConfig('PreCheckMajorVersionUpgrade')
      return self._RunMethod(
          config, request, global_params=global_params)

    PreCheckMajorVersionUpgrade.method_config = lambda: base_api.ApiMethodInfo(
        http_method='POST',
        method_id='sql.instances.preCheckMajorVersionUpgrade',
        ordered_params=['project', 'instance'],
        path_params=['instance', 'project'],
        query_params=[],
        relative_path='sql/v1beta4/projects/{project}/instances/{instance}/preCheckMajorVersionUpgrade',
        request_field='instancesPreCheckMajorVersionUpgradeRequest',
        request_type_name='SqlInstancesPreCheckMajorVersionUpgradeRequest',
        response_type_name='Operation',
        supports_download=False,
    )

    def PromoteReplica(self, request, global_params=None):
      r"""Promotes the read replica instance to be an independent Cloud SQL primary instance. Using this operation might cause your instance to restart.

      Args:
        request: (SqlInstancesPromoteReplicaRequest) input message
        global_params: (StandardQueryParameters, default: None) global arguments
      Returns:
        (Operation) The response message.
      """
      config = self.GetMethodConfig('PromoteReplica')
      return self._RunMethod(
          config, request, global_params=global_params)

    PromoteReplica.method_config = lambda: base_api.ApiMethodInfo(
        http_method='POST',
        method_id='sql.instances.promoteReplica',
        ordered_params=['project', 'instance'],
        path_params=['instance', 'project'],
        query_params=['failover'],
        relative_path='sql/v1beta4/projects/{project}/instances/{instance}/promoteReplica',
        request_field='',
        request_type_name='SqlInstancesPromoteReplicaRequest',
        response_type_name='Operation',
        supports_download=False,
    )

    def Reencrypt(self, request, global_params=None):
      r"""Reencrypt CMEK instance with latest key version.

      Args:
        request: (SqlInstancesReencryptRequest) input message
        global_params: (StandardQueryParameters, default: None) global arguments
      Returns:
        (Operation) The response message.
      """
      config = self.GetMethodConfig('Reencrypt')
      return self._RunMethod(
          config, request, global_params=global_params)

    Reencrypt.method_config = lambda: base_api.ApiMethodInfo(
        http_method='POST',
        method_id='sql.instances.reencrypt',
        ordered_params=['project', 'instance'],
        path_params=['instance', 'project'],
        query_params=[],
        relative_path='sql/v1beta4/projects/{project}/instances/{instance}/reencrypt',
        request_field='instancesReencryptRequest',
        request_type_name='SqlInstancesReencryptRequest',
        response_type_name='Operation',
        supports_download=False,
    )

    def ReleaseSsrsLease(self, request, global_params=None):
      r"""Release a lease for the setup of SQL Server Reporting Services (SSRS).

      Args:
        request: (SqlInstancesReleaseSsrsLeaseRequest) input message
        global_params: (StandardQueryParameters, default: None) global arguments
      Returns:
        (SqlInstancesReleaseSsrsLeaseResponse) The response message.
      """
      config = self.GetMethodConfig('ReleaseSsrsLease')
      return self._RunMethod(
          config, request, global_params=global_params)

    ReleaseSsrsLease.method_config = lambda: base_api.ApiMethodInfo(
        http_method='POST',
        method_id='sql.instances.releaseSsrsLease',
        ordered_params=['project', 'instance'],
        path_params=['instance', 'project'],
        query_params=[],
        relative_path='sql/v1beta4/projects/{project}/instances/{instance}/releaseSsrsLease',
        request_field='',
        request_type_name='SqlInstancesReleaseSsrsLeaseRequest',
        response_type_name='SqlInstancesReleaseSsrsLeaseResponse',
        supports_download=False,
    )

    def RemoveReplicationSource(self, request, global_params=None):
      r"""Remove source instance name to replicate from. Instance must not already be replicating from the designated source. Only applicable to external server writable destinations.

      Args:
        request: (SqlInstancesRemoveReplicationSourceRequest) input message
        global_params: (StandardQueryParameters, default: None) global arguments
      Returns:
        (Operation) The response message.
      """
      config = self.GetMethodConfig('RemoveReplicationSource')
      return self._RunMethod(
          config, request, global_params=global_params)

    RemoveReplicationSource.method_config = lambda: base_api.ApiMethodInfo(
        http_method='POST',
        method_id='sql.instances.removeReplicationSource',
        ordered_params=['project', 'instance'],
        path_params=['instance', 'project'],
        query_params=[],
        relative_path='sql/v1beta4/projects/{project}/instances/{instance}/removeReplicationSource',
        request_field='sqlInstancesRemoveReplicationSourceRequest',
        request_type_name='SqlInstancesRemoveReplicationSourceRequest',
        response_type_name='Operation',
        supports_download=False,
    )

    def ResetSslConfig(self, request, global_params=None):
      r"""Deletes all client certificates and generates a new server SSL certificate for the instance.

      Args:
        request: (SqlInstancesResetSslConfigRequest) input message
        global_params: (StandardQueryParameters, default: None) global arguments
      Returns:
        (Operation) The response message.
      """
      config = self.GetMethodConfig('ResetSslConfig')
      return self._RunMethod(
          config, request, global_params=global_params)

    ResetSslConfig.method_config = lambda: base_api.ApiMethodInfo(
        http_method='POST',
        method_id='sql.instances.resetSslConfig',
        ordered_params=['project', 'instance'],
        path_params=['instance', 'project'],
        query_params=['mode'],
        relative_path='sql/v1beta4/projects/{project}/instances/{instance}/resetSslConfig',
        request_field='',
        request_type_name='SqlInstancesResetSslConfigRequest',
        response_type_name='Operation',
        supports_download=False,
    )

    def Restart(self, request, global_params=None):
      r"""Restarts a Cloud SQL instance.

      Args:
        request: (SqlInstancesRestartRequest) input message
        global_params: (StandardQueryParameters, default: None) global arguments
      Returns:
        (Operation) The response message.
      """
      config = self.GetMethodConfig('Restart')
      return self._RunMethod(
          config, request, global_params=global_params)

    Restart.method_config = lambda: base_api.ApiMethodInfo(
        http_method='POST',
        method_id='sql.instances.restart',
        ordered_params=['project', 'instance'],
        path_params=['instance', 'project'],
        query_params=[],
        relative_path='sql/v1beta4/projects/{project}/instances/{instance}/restart',
        request_field='',
        request_type_name='SqlInstancesRestartRequest',
        response_type_name='Operation',
        supports_download=False,
    )

    def RestoreBackup(self, request, global_params=None):
      r"""Restores a backup of a Cloud SQL instance. Using this operation might cause your instance to restart.

      Args:
        request: (SqlInstancesRestoreBackupRequest) input message
        global_params: (StandardQueryParameters, default: None) global arguments
      Returns:
        (Operation) The response message.
      """
      config = self.GetMethodConfig('RestoreBackup')
      return self._RunMethod(
          config, request, global_params=global_params)

    RestoreBackup.method_config = lambda: base_api.ApiMethodInfo(
        http_method='POST',
        method_id='sql.instances.restoreBackup',
        ordered_params=['project', 'instance'],
        path_params=['instance', 'project'],
        query_params=[],
        relative_path='sql/v1beta4/projects/{project}/instances/{instance}/restoreBackup',
        request_field='instancesRestoreBackupRequest',
        request_type_name='SqlInstancesRestoreBackupRequest',
        response_type_name='Operation',
        supports_download=False,
    )

    def RotateServerCa(self, request, global_params=None):
      r"""Rotates the server certificate to one signed by the Certificate Authority (CA) version previously added with the addServerCA method. For instances that have enabled Certificate Authority Service (CAS) based server CA, use RotateServerCertificate to rotate the server certificate.

      Args:
        request: (SqlInstancesRotateServerCaRequest) input message
        global_params: (StandardQueryParameters, default: None) global arguments
      Returns:
        (Operation) The response message.
      """
      config = self.GetMethodConfig('RotateServerCa')
      return self._RunMethod(
          config, request, global_params=global_params)

    RotateServerCa.method_config = lambda: base_api.ApiMethodInfo(
        http_method='POST',
        method_id='sql.instances.rotateServerCa',
        ordered_params=['project', 'instance'],
        path_params=['instance', 'project'],
        query_params=[],
        relative_path='sql/v1beta4/projects/{project}/instances/{instance}/rotateServerCa',
        request_field='instancesRotateServerCaRequest',
        request_type_name='SqlInstancesRotateServerCaRequest',
        response_type_name='Operation',
        supports_download=False,
    )

    def StartReplica(self, request, global_params=None):
      r"""Starts the replication in the read replica instance.

      Args:
        request: (SqlInstancesStartReplicaRequest) input message
        global_params: (StandardQueryParameters, default: None) global arguments
      Returns:
        (Operation) The response message.
      """
      config = self.GetMethodConfig('StartReplica')
      return self._RunMethod(
          config, request, global_params=global_params)

    StartReplica.method_config = lambda: base_api.ApiMethodInfo(
        http_method='POST',
        method_id='sql.instances.startReplica',
        ordered_params=['project', 'instance'],
        path_params=['instance', 'project'],
        query_params=[],
        relative_path='sql/v1beta4/projects/{project}/instances/{instance}/startReplica',
        request_field='',
        request_type_name='SqlInstancesStartReplicaRequest',
        response_type_name='Operation',
        supports_download=False,
    )

    def StopReplica(self, request, global_params=None):
      r"""Stops the replication in the read replica instance.

      Args:
        request: (SqlInstancesStopReplicaRequest) input message
        global_params: (StandardQueryParameters, default: None) global arguments
      Returns:
        (Operation) The response message.
      """
      config = self.GetMethodConfig('StopReplica')
      return self._RunMethod(
          config, request, global_params=global_params)

    StopReplica.method_config = lambda: base_api.ApiMethodInfo(
        http_method='POST',
        method_id='sql.instances.stopReplica',
        ordered_params=['project', 'instance'],
        path_params=['instance', 'project'],
        query_params=[],
        relative_path='sql/v1beta4/projects/{project}/instances/{instance}/stopReplica',
        request_field='',
        request_type_name='SqlInstancesStopReplicaRequest',
        response_type_name='Operation',
        supports_download=False,
    )

    def Switchover(self, request, global_params=None):
      r"""Switches over from the primary instance to the DR replica instance.

      Args:
        request: (SqlInstancesSwitchoverRequest) input message
        global_params: (StandardQueryParameters, default: None) global arguments
      Returns:
        (Operation) The response message.
      """
      config = self.GetMethodConfig('Switchover')
      return self._RunMethod(
          config, request, global_params=global_params)

    Switchover.method_config = lambda: base_api.ApiMethodInfo(
        http_method='POST',
        method_id='sql.instances.switchover',
        ordered_params=['project', 'instance'],
        path_params=['instance', 'project'],
        query_params=['dbTimeout'],
        relative_path='sql/v1beta4/projects/{project}/instances/{instance}/switchover',
        request_field='',
        request_type_name='SqlInstancesSwitchoverRequest',
        response_type_name='Operation',
        supports_download=False,
    )

    def TruncateLog(self, request, global_params=None):
      r"""Truncate MySQL general and slow query log tables MySQL only.

      Args:
        request: (SqlInstancesTruncateLogRequest) input message
        global_params: (StandardQueryParameters, default: None) global arguments
      Returns:
        (Operation) The response message.
      """
      config = self.GetMethodConfig('TruncateLog')
      return self._RunMethod(
          config, request, global_params=global_params)

    TruncateLog.method_config = lambda: base_api.ApiMethodInfo(
        http_method='POST',
        method_id='sql.instances.truncateLog',
        ordered_params=['project', 'instance'],
        path_params=['instance', 'project'],
        query_params=[],
        relative_path='sql/v1beta4/projects/{project}/instances/{instance}/truncateLog',
        request_field='instancesTruncateLogRequest',
        request_type_name='SqlInstancesTruncateLogRequest',
        response_type_name='Operation',
        supports_download=False,
    )

    def Update(self, request, global_params=None):
      r"""Updates settings of a Cloud SQL instance. Using this operation might cause your instance to restart.

      Args:
        request: (SqlInstancesUpdateRequest) input message
        global_params: (StandardQueryParameters, default: None) global arguments
      Returns:
        (Operation) The response message.
      """
      config = self.GetMethodConfig('Update')
      return self._RunMethod(
          config, request, global_params=global_params)

    Update.method_config = lambda: base_api.ApiMethodInfo(
        http_method='PUT',
        method_id='sql.instances.update',
        ordered_params=['project', 'instance'],
        path_params=['instance', 'project'],
        query_params=['enforcePsaWriteEndpoint'],
        relative_path='sql/v1beta4/projects/{project}/instances/{instance}',
        request_field='databaseInstance',
        request_type_name='SqlInstancesUpdateRequest',
        response_type_name='Operation',
        supports_download=False,
    )

  class OperationsService(base_api.BaseApiService):
    """Service class for the operations resource."""

    _NAME = 'operations'

    def __init__(self, client):
      super(SqladminV1beta4.OperationsService, self).__init__(client)
      self._upload_configs = {
          }

    def Cancel(self, request, global_params=None):
      r"""Cancels an instance operation that has been performed on an instance. Ordinarily, this method name should be `CancelSqlOperation`.

      Args:
        request: (SqlOperationsCancelRequest) input message
        global_params: (StandardQueryParameters, default: None) global arguments
      Returns:
        (Empty) The response message.
      """
      config = self.GetMethodConfig('Cancel')
      return self._RunMethod(
          config, request, global_params=global_params)

    Cancel.method_config = lambda: base_api.ApiMethodInfo(
        http_method='POST',
        method_id='sql.operations.cancel',
        ordered_params=['project', 'operation'],
        path_params=['operation', 'project'],
        query_params=[],
        relative_path='sql/v1beta4/projects/{project}/operations/{operation}/cancel',
        request_field='',
        request_type_name='SqlOperationsCancelRequest',
        response_type_name='Empty',
        supports_download=False,
    )

    def Get(self, request, global_params=None):
      r"""Retrieves an instance operation that has been performed on an instance.

      Args:
        request: (SqlOperationsGetRequest) input message
        global_params: (StandardQueryParameters, default: None) global arguments
      Returns:
        (Operation) The response message.
      """
      config = self.GetMethodConfig('Get')
      return self._RunMethod(
          config, request, global_params=global_params)

    Get.method_config = lambda: base_api.ApiMethodInfo(
        http_method='GET',
        method_id='sql.operations.get',
        ordered_params=['project', 'operation'],
        path_params=['operation', 'project'],
        query_params=[],
        relative_path='sql/v1beta4/projects/{project}/operations/{operation}',
        request_field='',
        request_type_name='SqlOperationsGetRequest',
        response_type_name='Operation',
        supports_download=False,
    )

    def List(self, request, global_params=None):
      r"""Lists all instance operations that have been performed on the given Cloud SQL instance in the reverse chronological order of the start time.

      Args:
        request: (SqlOperationsListRequest) input message
        global_params: (StandardQueryParameters, default: None) global arguments
      Returns:
        (OperationsListResponse) The response message.
      """
      config = self.GetMethodConfig('List')
      return self._RunMethod(
          config, request, global_params=global_params)

    List.method_config = lambda: base_api.ApiMethodInfo(
        http_method='GET',
        method_id='sql.operations.list',
        ordered_params=['project'],
        path_params=['project'],
        query_params=['filter', 'instance', 'maxResults', 'pageToken'],
        relative_path='sql/v1beta4/projects/{project}/operations',
        request_field='',
        request_type_name='SqlOperationsListRequest',
        response_type_name='OperationsListResponse',
        supports_download=False,
    )

  class ProjectsInstancesService(base_api.BaseApiService):
    """Service class for the projects_instances resource."""

    _NAME = 'projects_instances'

    def __init__(self, client):
      super(SqladminV1beta4.ProjectsInstancesService, self).__init__(client)
      self._upload_configs = {
          }

    def GetDiskShrinkConfig(self, request, global_params=None):
      r"""Get Disk Shrink Config for a given instance.

      Args:
        request: (SqlProjectsInstancesGetDiskShrinkConfigRequest) input message
        global_params: (StandardQueryParameters, default: None) global arguments
      Returns:
        (SqlInstancesGetDiskShrinkConfigResponse) The response message.
      """
      config = self.GetMethodConfig('GetDiskShrinkConfig')
      return self._RunMethod(
          config, request, global_params=global_params)

    GetDiskShrinkConfig.method_config = lambda: base_api.ApiMethodInfo(
        http_method='GET',
        method_id='sql.projects.instances.getDiskShrinkConfig',
        ordered_params=['project', 'instance'],
        path_params=['instance', 'project'],
        query_params=[],
        relative_path='sql/v1beta4/projects/{project}/instances/{instance}/getDiskShrinkConfig',
        request_field='',
        request_type_name='SqlProjectsInstancesGetDiskShrinkConfigRequest',
        response_type_name='SqlInstancesGetDiskShrinkConfigResponse',
        supports_download=False,
    )

    def GetLatestRecoveryTime(self, request, global_params=None):
      r"""Get Latest Recovery Time for a given instance.

      Args:
        request: (SqlProjectsInstancesGetLatestRecoveryTimeRequest) input message
        global_params: (StandardQueryParameters, default: None) global arguments
      Returns:
        (SqlInstancesGetLatestRecoveryTimeResponse) The response message.
      """
      config = self.GetMethodConfig('GetLatestRecoveryTime')
      return self._RunMethod(
          config, request, global_params=global_params)

    GetLatestRecoveryTime.method_config = lambda: base_api.ApiMethodInfo(
        http_method='GET',
        method_id='sql.projects.instances.getLatestRecoveryTime',
        ordered_params=['project', 'instance'],
        path_params=['instance', 'project'],
        query_params=['sourceInstanceDeletionTime'],
        relative_path='sql/v1beta4/projects/{project}/instances/{instance}/getLatestRecoveryTime',
        request_field='',
        request_type_name='SqlProjectsInstancesGetLatestRecoveryTimeRequest',
        response_type_name='SqlInstancesGetLatestRecoveryTimeResponse',
        supports_download=False,
    )

    def PerformDiskShrink(self, request, global_params=None):
      r"""Perform Disk Shrink on primary instance.

      Args:
        request: (SqlProjectsInstancesPerformDiskShrinkRequest) input message
        global_params: (StandardQueryParameters, default: None) global arguments
      Returns:
        (Operation) The response message.
      """
      config = self.GetMethodConfig('PerformDiskShrink')
      return self._RunMethod(
          config, request, global_params=global_params)

    PerformDiskShrink.method_config = lambda: base_api.ApiMethodInfo(
        http_method='POST',
        method_id='sql.projects.instances.performDiskShrink',
        ordered_params=['project', 'instance'],
        path_params=['instance', 'project'],
        query_params=[],
        relative_path='sql/v1beta4/projects/{project}/instances/{instance}/performDiskShrink',
        request_field='performDiskShrinkContext',
        request_type_name='SqlProjectsInstancesPerformDiskShrinkRequest',
        response_type_name='Operation',
        supports_download=False,
    )

    def RescheduleMaintenance(self, request, global_params=None):
      r"""Reschedules the maintenance on the given instance.

      Args:
        request: (SqlProjectsInstancesRescheduleMaintenanceRequest) input message
        global_params: (StandardQueryParameters, default: None) global arguments
      Returns:
        (Operation) The response message.
      """
      config = self.GetMethodConfig('RescheduleMaintenance')
      return self._RunMethod(
          config, request, global_params=global_params)

    RescheduleMaintenance.method_config = lambda: base_api.ApiMethodInfo(
        http_method='POST',
        method_id='sql.projects.instances.rescheduleMaintenance',
        ordered_params=['project', 'instance'],
        path_params=['instance', 'project'],
        query_params=[],
        relative_path='sql/v1beta4/projects/{project}/instances/{instance}/rescheduleMaintenance',
        request_field='sqlInstancesRescheduleMaintenanceRequestBody',
        request_type_name='SqlProjectsInstancesRescheduleMaintenanceRequest',
        response_type_name='Operation',
        supports_download=False,
    )

    def ResetReplicaSize(self, request, global_params=None):
      r"""Reset Replica Size to primary instance disk size.

      Args:
        request: (SqlProjectsInstancesResetReplicaSizeRequest) input message
        global_params: (StandardQueryParameters, default: None) global arguments
      Returns:
        (Operation) The response message.
      """
      config = self.GetMethodConfig('ResetReplicaSize')
      return self._RunMethod(
          config, request, global_params=global_params)

    ResetReplicaSize.method_config = lambda: base_api.ApiMethodInfo(
        http_method='POST',
        method_id='sql.projects.instances.resetReplicaSize',
        ordered_params=['project', 'instance'],
        path_params=['instance', 'project'],
        query_params=[],
        relative_path='sql/v1beta4/projects/{project}/instances/{instance}/resetReplicaSize',
        request_field='sqlInstancesResetReplicaSizeRequest',
        request_type_name='SqlProjectsInstancesResetReplicaSizeRequest',
        response_type_name='Operation',
        supports_download=False,
    )

    def StartExternalSync(self, request, global_params=None):
      r"""Start External primary instance migration.

      Args:
        request: (SqlProjectsInstancesStartExternalSyncRequest) input message
        global_params: (StandardQueryParameters, default: None) global arguments
      Returns:
        (Operation) The response message.
      """
      config = self.GetMethodConfig('StartExternalSync')
      return self._RunMethod(
          config, request, global_params=global_params)

    StartExternalSync.method_config = lambda: base_api.ApiMethodInfo(
        http_method='POST',
        method_id='sql.projects.instances.startExternalSync',
        ordered_params=['project', 'instance'],
        path_params=['instance', 'project'],
        query_params=[],
        relative_path='sql/v1beta4/projects/{project}/instances/{instance}/startExternalSync',
        request_field='sqlInstancesStartExternalSyncRequest',
        request_type_name='SqlProjectsInstancesStartExternalSyncRequest',
        response_type_name='Operation',
        supports_download=False,
    )

    def VerifyExternalSyncSettings(self, request, global_params=None):
      r"""Verify External primary instance external sync settings.

      Args:
        request: (SqlProjectsInstancesVerifyExternalSyncSettingsRequest) input message
        global_params: (StandardQueryParameters, default: None) global arguments
      Returns:
        (SqlInstancesVerifyExternalSyncSettingsResponse) The response message.
      """
      config = self.GetMethodConfig('VerifyExternalSyncSettings')
      return self._RunMethod(
          config, request, global_params=global_params)

    VerifyExternalSyncSettings.method_config = lambda: base_api.ApiMethodInfo(
        http_method='POST',
        method_id='sql.projects.instances.verifyExternalSyncSettings',
        ordered_params=['project', 'instance'],
        path_params=['instance', 'project'],
        query_params=[],
        relative_path='sql/v1beta4/projects/{project}/instances/{instance}/verifyExternalSyncSettings',
        request_field='sqlInstancesVerifyExternalSyncSettingsRequest',
        request_type_name='SqlProjectsInstancesVerifyExternalSyncSettingsRequest',
        response_type_name='SqlInstancesVerifyExternalSyncSettingsResponse',
        supports_download=False,
    )

  class ProjectsService(base_api.BaseApiService):
    """Service class for the projects resource."""

    _NAME = 'projects'

    def __init__(self, client):
      super(SqladminV1beta4.ProjectsService, self).__init__(client)
      self._upload_configs = {
          }

  class SslCertsService(base_api.BaseApiService):
    """Service class for the sslCerts resource."""

    _NAME = 'sslCerts'

    def __init__(self, client):
      super(SqladminV1beta4.SslCertsService, self).__init__(client)
      self._upload_configs = {
          }

    def CreateEphemeral(self, request, global_params=None):
      r"""Generates a short-lived X509 certificate containing the provided public key and signed by a private key specific to the target instance. Users may use the certificate to authenticate as themselves when connecting to the database.

      Args:
        request: (SqlSslCertsCreateEphemeralRequest) input message
        global_params: (StandardQueryParameters, default: None) global arguments
      Returns:
        (SslCert) The response message.
      """
      config = self.GetMethodConfig('CreateEphemeral')
      return self._RunMethod(
          config, request, global_params=global_params)

    CreateEphemeral.method_config = lambda: base_api.ApiMethodInfo(
        http_method='POST',
        method_id='sql.sslCerts.createEphemeral',
        ordered_params=['project', 'instance'],
        path_params=['instance', 'project'],
        query_params=[],
        relative_path='sql/v1beta4/projects/{project}/instances/{instance}/createEphemeral',
        request_field='sslCertsCreateEphemeralRequest',
        request_type_name='SqlSslCertsCreateEphemeralRequest',
        response_type_name='SslCert',
        supports_download=False,
    )

    def Delete(self, request, global_params=None):
      r"""Deletes the SSL certificate. For First Generation instances, the certificate remains valid until the instance is restarted.

      Args:
        request: (SqlSslCertsDeleteRequest) input message
        global_params: (StandardQueryParameters, default: None) global arguments
      Returns:
        (Operation) The response message.
      """
      config = self.GetMethodConfig('Delete')
      return self._RunMethod(
          config, request, global_params=global_params)

    Delete.method_config = lambda: base_api.ApiMethodInfo(
        http_method='DELETE',
        method_id='sql.sslCerts.delete',
        ordered_params=['project', 'instance', 'sha1Fingerprint'],
        path_params=['instance', 'project', 'sha1Fingerprint'],
        query_params=[],
        relative_path='sql/v1beta4/projects/{project}/instances/{instance}/sslCerts/{sha1Fingerprint}',
        request_field='',
        request_type_name='SqlSslCertsDeleteRequest',
        response_type_name='Operation',
        supports_download=False,
    )

    def Get(self, request, global_params=None):
      r"""Retrieves a particular SSL certificate. Does not include the private key (required for usage). The private key must be saved from the response to initial creation.

      Args:
        request: (SqlSslCertsGetRequest) input message
        global_params: (StandardQueryParameters, default: None) global arguments
      Returns:
        (SslCert) The response message.
      """
      config = self.GetMethodConfig('Get')
      return self._RunMethod(
          config, request, global_params=global_params)

    Get.method_config = lambda: base_api.ApiMethodInfo(
        http_method='GET',
        method_id='sql.sslCerts.get',
        ordered_params=['project', 'instance', 'sha1Fingerprint'],
        path_params=['instance', 'project', 'sha1Fingerprint'],
        query_params=[],
        relative_path='sql/v1beta4/projects/{project}/instances/{instance}/sslCerts/{sha1Fingerprint}',
        request_field='',
        request_type_name='SqlSslCertsGetRequest',
        response_type_name='SslCert',
        supports_download=False,
    )

    def Insert(self, request, global_params=None):
      r"""Creates an SSL certificate and returns it along with the private key and server certificate authority. The new certificate will not be usable until the instance is restarted.

      Args:
        request: (SqlSslCertsInsertRequest) input message
        global_params: (StandardQueryParameters, default: None) global arguments
      Returns:
        (SslCertsInsertResponse) The response message.
      """
      config = self.GetMethodConfig('Insert')
      return self._RunMethod(
          config, request, global_params=global_params)

    Insert.method_config = lambda: base_api.ApiMethodInfo(
        http_method='POST',
        method_id='sql.sslCerts.insert',
        ordered_params=['project', 'instance'],
        path_params=['instance', 'project'],
        query_params=[],
        relative_path='sql/v1beta4/projects/{project}/instances/{instance}/sslCerts',
        request_field='sslCertsInsertRequest',
        request_type_name='SqlSslCertsInsertRequest',
        response_type_name='SslCertsInsertResponse',
        supports_download=False,
    )

    def List(self, request, global_params=None):
      r"""Lists all of the current SSL certificates for the instance.

      Args:
        request: (SqlSslCertsListRequest) input message
        global_params: (StandardQueryParameters, default: None) global arguments
      Returns:
        (SslCertsListResponse) The response message.
      """
      config = self.GetMethodConfig('List')
      return self._RunMethod(
          config, request, global_params=global_params)

    List.method_config = lambda: base_api.ApiMethodInfo(
        http_method='GET',
        method_id='sql.sslCerts.list',
        ordered_params=['project', 'instance'],
        path_params=['instance', 'project'],
        query_params=[],
        relative_path='sql/v1beta4/projects/{project}/instances/{instance}/sslCerts',
        request_field='',
        request_type_name='SqlSslCertsListRequest',
        response_type_name='SslCertsListResponse',
        supports_download=False,
    )

  class TiersService(base_api.BaseApiService):
    """Service class for the tiers resource."""

    _NAME = 'tiers'

    def __init__(self, client):
      super(SqladminV1beta4.TiersService, self).__init__(client)
      self._upload_configs = {
          }

    def List(self, request, global_params=None):
      r"""Lists all available machine types (tiers) for Cloud SQL, for example, `db-custom-1-3840`. For related information, see [Pricing](/sql/pricing).

      Args:
        request: (SqlTiersListRequest) input message
        global_params: (StandardQueryParameters, default: None) global arguments
      Returns:
        (TiersListResponse) The response message.
      """
      config = self.GetMethodConfig('List')
      return self._RunMethod(
          config, request, global_params=global_params)

    List.method_config = lambda: base_api.ApiMethodInfo(
        http_method='GET',
        method_id='sql.tiers.list',
        ordered_params=['project'],
        path_params=['project'],
        query_params=[],
        relative_path='sql/v1beta4/projects/{project}/tiers',
        request_field='',
        request_type_name='SqlTiersListRequest',
        response_type_name='TiersListResponse',
        supports_download=False,
    )

  class UsersService(base_api.BaseApiService):
    """Service class for the users resource."""

    _NAME = 'users'

    def __init__(self, client):
      super(SqladminV1beta4.UsersService, self).__init__(client)
      self._upload_configs = {
          }

    def Delete(self, request, global_params=None):
      r"""Deletes a user from a Cloud SQL instance.

      Args:
        request: (SqlUsersDeleteRequest) input message
        global_params: (StandardQueryParameters, default: None) global arguments
      Returns:
        (Operation) The response message.
      """
      config = self.GetMethodConfig('Delete')
      return self._RunMethod(
          config, request, global_params=global_params)

    Delete.method_config = lambda: base_api.ApiMethodInfo(
        http_method='DELETE',
        method_id='sql.users.delete',
        ordered_params=['project', 'instance'],
        path_params=['instance', 'project'],
        query_params=['host', 'name'],
        relative_path='sql/v1beta4/projects/{project}/instances/{instance}/users',
        request_field='',
        request_type_name='SqlUsersDeleteRequest',
        response_type_name='Operation',
        supports_download=False,
    )

    def Get(self, request, global_params=None):
      r"""Retrieves a resource containing information about a user.

      Args:
        request: (SqlUsersGetRequest) input message
        global_params: (StandardQueryParameters, default: None) global arguments
      Returns:
        (User) The response message.
      """
      config = self.GetMethodConfig('Get')
      return self._RunMethod(
          config, request, global_params=global_params)

    Get.method_config = lambda: base_api.ApiMethodInfo(
        http_method='GET',
        method_id='sql.users.get',
        ordered_params=['project', 'instance', 'name'],
        path_params=['instance', 'name', 'project'],
        query_params=['host'],
        relative_path='sql/v1beta4/projects/{project}/instances/{instance}/users/{name}',
        request_field='',
        request_type_name='SqlUsersGetRequest',
        response_type_name='User',
        supports_download=False,
    )

    def Insert(self, request, global_params=None):
      r"""Creates a new user in a Cloud SQL instance.

      Args:
        request: (User) input message
        global_params: (StandardQueryParameters, default: None) global arguments
      Returns:
        (Operation) The response message.
      """
      config = self.GetMethodConfig('Insert')
      return self._RunMethod(
          config, request, global_params=global_params)

    Insert.method_config = lambda: base_api.ApiMethodInfo(
        http_method='POST',
        method_id='sql.users.insert',
        ordered_params=['project', 'instance'],
        path_params=['instance', 'project'],
        query_params=[],
        relative_path='sql/v1beta4/projects/{project}/instances/{instance}/users',
        request_field='<request>',
        request_type_name='User',
        response_type_name='Operation',
        supports_download=False,
    )

    def List(self, request, global_params=None):
      r"""Lists users in the specified Cloud SQL instance.

      Args:
        request: (SqlUsersListRequest) input message
        global_params: (StandardQueryParameters, default: None) global arguments
      Returns:
        (UsersListResponse) The response message.
      """
      config = self.GetMethodConfig('List')
      return self._RunMethod(
          config, request, global_params=global_params)

    List.method_config = lambda: base_api.ApiMethodInfo(
        http_method='GET',
        method_id='sql.users.list',
        ordered_params=['project', 'instance'],
        path_params=['instance', 'project'],
        query_params=[],
        relative_path='sql/v1beta4/projects/{project}/instances/{instance}/users',
        request_field='',
        request_type_name='SqlUsersListRequest',
        response_type_name='UsersListResponse',
        supports_download=False,
    )

    def Update(self, request, global_params=None):
      r"""Updates an existing user in a Cloud SQL instance.

      Args:
        request: (SqlUsersUpdateRequest) input message
        global_params: (StandardQueryParameters, default: None) global arguments
      Returns:
        (Operation) The response message.
      """
      config = self.GetMethodConfig('Update')
      return self._RunMethod(
          config, request, global_params=global_params)

    Update.method_config = lambda: base_api.ApiMethodInfo(
        http_method='PUT',
        method_id='sql.users.update',
        ordered_params=['project', 'instance'],
        path_params=['instance', 'project'],
        query_params=['databaseRoles', 'host', 'name', 'revokeExistingRoles', 'revokeExistingServerRoles', 'serverRoles'],
        relative_path='sql/v1beta4/projects/{project}/instances/{instance}/users',
        request_field='user',
        request_type_name='SqlUsersUpdateRequest',
        response_type_name='Operation',
        supports_download=False,
    )
